@@ -9,6 +9,7 @@
 
 #include <algorithm>
 #include <array>
+#include <bit>
 #include <cstring>
 #include <string>
 #include <vector>
@@ -17,7 +18,6 @@
 #include <windows.h>
 #endif
 
-#include "Common/BitUtils.h"
 #include "Common/CommonTypes.h"
 #include "Common/MsgHandler.h"
 #include "Common/StringUtil.h"
@@ -249,26 +249,26 @@ static void unscramble1(u32* addr, u32* val)
 {
   u32 tmp;
 
-  *val = Common::RotateLeft(*val, 4);
+  *val = std::rotl(*val, 4);
 
   tmp = ((*addr ^ *val) & 0xF0F0F0F0);
   *addr ^= tmp;
-  *val = Common::RotateRight((*val ^ tmp), 0x14);
+  *val = std::rotr((*val ^ tmp), 0x14);
 
   tmp = ((*addr ^ *val) & 0xFFFF0000);
   *addr ^= tmp;
-  *val = Common::RotateRight((*val ^ tmp), 0x12);
+  *val = std::rotr((*val ^ tmp), 0x12);
 
   tmp = ((*addr ^ *val) & 0x33333333);
   *addr ^= tmp;
-  *val = Common::RotateRight((*val ^ tmp), 6);
+  *val = std::rotr((*val ^ tmp), 6);
 
   tmp = ((*addr ^ *val) & 0x00FF00FF);
   *addr ^= tmp;
-  *val = Common::RotateLeft((*val ^ tmp), 9);
+  *val = std::rotl((*val ^ tmp), 9);
 
   tmp = ((*addr ^ *val) & 0xAAAAAAAA);
-  *addr = Common::RotateLeft((*addr ^ tmp), 1);
+  *addr = std::rotl((*addr ^ tmp), 1);
   *val ^= tmp;
 }
 
@@ -276,27 +276,27 @@ static void unscramble2(u32* addr, u32* val)
 {
   u32 tmp;
 
-  *val = Common::RotateRight(*val, 1);
+  *val = std::rotr(*val, 1);
 
   tmp = ((*addr ^ *val) & 0xAAAAAAAA);
   *val ^= tmp;
-  *addr = Common::RotateRight((*addr ^ tmp), 9);
+  *addr = std::rotr((*addr ^ tmp), 9);
 
   tmp = ((*addr ^ *val) & 0x00FF00FF);
   *val ^= tmp;
-  *addr = Common::RotateLeft((*addr ^ tmp), 6);
+  *addr = std::rotl((*addr ^ tmp), 6);
 
   tmp = ((*addr ^ *val) & 0x33333333);
   *val ^= tmp;
-  *addr = Common::RotateLeft((*addr ^ tmp), 0x12);
+  *addr = std::rotl((*addr ^ tmp), 0x12);
 
   tmp = ((*addr ^ *val) & 0xFFFF0000);
   *val ^= tmp;
-  *addr = Common::RotateLeft((*addr ^ tmp), 0x14);
+  *addr = std::rotl((*addr ^ tmp), 0x14);
 
   tmp = ((*addr ^ *val) & 0xF0F0F0F0);
   *val ^= tmp;
-  *addr = Common::RotateRight((*addr ^ tmp), 4);
+  *addr = std::rotr((*addr ^ tmp), 4);
 }
 
 static void decryptcode(const u32* seeds, u32* code)
@@ -309,13 +309,13 @@ static void decryptcode(const u32* seeds, u32* code)
   unscramble1(&addr, &val);
   while (i < 32)
   {
-    tmp = (Common::RotateRight(val, 4) ^ seeds[i++]);
+    tmp = (std::rotr(val, 4) ^ seeds[i++]);
     tmp2 = (val ^ seeds[i++]);
     addr ^= (table6[tmp & 0x3F] ^ table4[(tmp >> 8) & 0x3F] ^ table2[(tmp >> 16) & 0x3F] ^
              table0[(tmp >> 24) & 0x3F] ^ table7[tmp2 & 0x3F] ^ table5[(tmp2 >> 8) & 0x3F] ^
              table3[(tmp2 >> 16) & 0x3F] ^ table1[(tmp2 >> 24) & 0x3F]);
 
-    tmp = (Common::RotateRight(addr, 4) ^ seeds[i++]);
+    tmp = (std::rotr(addr, 4) ^ seeds[i++]);
     tmp2 = (addr ^ seeds[i++]);
     val ^= (table6[tmp & 0x3F] ^ table4[(tmp >> 8) & 0x3F] ^ table2[(tmp >> 16) & 0x3F] ^
             table0[(tmp >> 24) & 0x3F] ^ table7[tmp2 & 0x3F] ^ table5[(tmp2 >> 8) & 0x3F] ^
