@@ -3,12 +3,6 @@
 
 #include "Core/Boot/Boot.h"
 
-#ifdef _MSC_VER
-#include <filesystem>
-namespace fs = std::filesystem;
-#define HAS_STD_FILESYSTEM
-#endif
-
 #include <algorithm>
 #include <array>
 #include <cstring>
@@ -21,7 +15,6 @@ namespace fs = std::filesystem;
 #include <vector>
 
 #include "Common/Align.h"
-#include "Common/CDUtils.h"
 #include "Common/CommonPaths.h"
 #include "Common/CommonTypes.h"
 #include "Common/Config/Config.h"
@@ -55,6 +48,7 @@ namespace fs = std::filesystem;
 #include "Core/PowerPC/PPCAnalyst.h"
 #include "Core/PowerPC/PPCSymbolDB.h"
 #include "Core/PowerPC/PowerPC.h"
+#include "Core/System.h"
 
 #include "DiscIO/Enums.h"
 #include "DiscIO/GameModDescriptor.h"
@@ -66,10 +60,6 @@ namespace fs = std::filesystem;
 static std::vector<std::string> ReadM3UFile(const std::string& m3u_path,
                                             const std::string& folder_path)
 {
-#ifndef HAS_STD_FILESYSTEM
-  ASSERT(folder_path.back() == '/');
-#endif
-
   std::vector<std::string> result;
   std::vector<std::string> nonexistent;
 
@@ -82,7 +72,7 @@ static std::vector<std::string> ReadM3UFile(const std::string& m3u_path,
     // This is the UTF-8 representation of U+FEFF.
     constexpr std::string_view utf8_bom = "\xEF\xBB\xBF";
 
-    if (StringBeginsWith(line, utf8_bom))
+    if (line.starts_with(utf8_bom))
     {
       WARN_LOG_FMT(BOOT, "UTF-8 BOM in file: {}", m3u_path);
       line.erase(0, utf8_bom.length());
@@ -90,12 +80,7 @@ static std::vector<std::string> ReadM3UFile(const std::string& m3u_path,
 
     if (!line.empty() && line.front() != '#')  // Comments start with #
     {
-#ifdef HAS_STD_FILESYSTEM
       const std::string path_to_add = PathToString(StringToPath(folder_path) / StringToPath(line));
-#else
-      const std::string path_to_add = line.front() != '/' ? folder_path + line : line;
-#endif
-
       (File::Exists(path_to_add) ? result : nonexistent).push_back(path_to_add);
     }
   }
@@ -204,10 +189,12 @@ std::unique_ptr<BootParameters> BootParameters::GenerateFromFile(std::vector<std
 {
   ASSERT(!paths.empty());
 
-  const bool is_drive = Common::IsCDROMDevice(paths.front());
+  for (std::string& path : paths)
+    UnifyPathSeparators(path);
+
   // Check if the file exist, we may have gotten it from a --elf command line
   // that gave an incorrect file name
-  if (!is_drive && !File::Exists(paths.front()))
+  if (!File::Exists(paths.front()))
   {
     PanicAlertFmtT("The specified file \"{0}\" does not exist", paths.front());
     return {};
@@ -224,6 +211,9 @@ std::unique_ptr<BootParameters> BootParameters::GenerateFromFile(std::vector<std
     if (paths.empty())
       return {};
 
+    for (std::string& path : paths)
+      UnifyPathSeparators(path);
+
     SplitPath(paths.front(), nullptr, nullptr, &extension);
     Common::ToLower(&extension);
   }
@@ -243,7 +233,7 @@ std::unique_ptr<BootParameters> BootParameters::GenerateFromFile(std::vector<std
 
   static const std::unordered_set<std::string> disc_image_extensions = {
       {".gcm", ".iso", ".tgc", ".wbfs", ".ciso", ".gcz", ".wia", ".rvz", ".nfs", ".dol", ".elf"}};
-  if (disc_image_extensions.find(extension) != disc_image_extensions.end() || is_drive)
+  if (disc_image_extensions.find(extension) != disc_image_extensions.end())
   {
     std::unique_ptr<DiscIO::VolumeDisc> disc = DiscIO::CreateDisc(path);
     if (disc)
@@ -266,18 +256,7 @@ std::unique_ptr<BootParameters> BootParameters::GenerateFromFile(std::vector<std
                                               std::move(boot_session_data_));
     }
 
-    if (is_drive)
-    {
-      PanicAlertFmtT("Could not read \"{0}\". "
-                     "There is no disc in the drive or it is not a GameCube/Wii backup. "
-                     "Please note that Dolphin cannot play games directly from the original "
-                     "GameCube and Wii discs.",
-                     path);
-    }
-    else
-    {
-      PanicAlertFmtT("\"{0}\" is an invalid GCM/ISO file, or is not a GC/Wii ISO.", path);
-    }
+    PanicAlertFmtT("\"{0}\" is an invalid GCM/ISO file, or is not a GC/Wii ISO.", path);
     return {};
   }
 
@@ -338,29 +317,34 @@ static const DiscIO::VolumeDisc* SetDisc(std::unique_ptr<DiscIO::VolumeDisc> dis
                                          std::vector<std::string> auto_disc_change_paths = {})
 {
   const DiscIO::VolumeDisc* pointer = disc.get();
-  DVDInterface::SetDisc(std::move(disc), auto_disc_change_paths);
+  Core::System::GetInstance().GetDVDInterface().SetDisc(std::move(disc), auto_disc_change_paths);
   return pointer;
 }
 
-bool CBoot::DVDRead(const DiscIO::VolumeDisc& disc, u64 dvd_offset, u32 output_address, u32 length,
-                    const DiscIO::Partition& partition)
+bool CBoot::DVDRead(Core::System& system, const DiscIO::VolumeDisc& disc, u64 dvd_offset,
+                    u32 output_address, u32 length, const DiscIO::Partition& partition)
 {
   std::vector<u8> buffer(length);
   if (!disc.Read(dvd_offset, length, buffer.data(), partition))
     return false;
-  Memory::CopyToEmu(output_address, buffer.data(), length);
+
+  auto& memory = system.GetMemory();
+  memory.CopyToEmu(output_address, buffer.data(), length);
   return true;
 }
 
-bool CBoot::DVDReadDiscID(const DiscIO::VolumeDisc& disc, u32 output_address)
+bool CBoot::DVDReadDiscID(Core::System& system, const DiscIO::VolumeDisc& disc, u32 output_address)
 {
   std::array<u8, 0x20> buffer;
   if (!disc.Read(0, buffer.size(), buffer.data(), DiscIO::PARTITION_NONE))
     return false;
-  Memory::CopyToEmu(output_address, buffer.data(), buffer.size());
+
+  auto& memory = system.GetMemory();
+  memory.CopyToEmu(output_address, buffer.data(), buffer.size());
+
   // Transition out of the DiscIdNotRead state (which the drive should be in at this point,
   // on the assumption that this is only used for the first read)
-  DVDInterface::SetDriveState(DVDInterface::DriveState::ReadyNoReadsMade);
+  system.GetDVDInterface().SetDriveState(DVD::DriveState::ReadyNoReadsMade);
   return true;
 }
 
@@ -389,11 +373,11 @@ bool CBoot::FindMapFile(std::string* existing_map_file, std::string* writable_ma
   return false;
 }
 
-bool CBoot::LoadMapFromFilename()
+bool CBoot::LoadMapFromFilename(const Core::CPUThreadGuard& guard)
 {
   std::string strMapFilename;
   bool found = FindMapFile(&strMapFilename, nullptr);
-  if (found && g_symbolDB.LoadMap(strMapFilename))
+  if (found && g_symbolDB.LoadMap(guard, strMapFilename))
   {
     UpdateDebugger_MapLoaded();
     return true;
@@ -405,7 +389,7 @@ bool CBoot::LoadMapFromFilename()
 // If ipl.bin is not found, this function does *some* of what BS1 does:
 // loading IPL(BS2) and jumping to it.
 // It does not initialize the hardware or anything else like BS1 does.
-bool CBoot::Load_BS2(const std::string& boot_rom_filename)
+bool CBoot::Load_BS2(Core::System& system, const std::string& boot_rom_filename)
 {
   // CRC32 hashes of the IPL file, obtained from Redump
   constexpr u32 NTSC_v1_0 = 0x6DAC1F2A;
@@ -455,25 +439,27 @@ bool CBoot::Load_BS2(const std::string& boot_rom_filename)
   // copying the initial boot code to 0x81200000 is a hack.
   // For now, HLE the first few instructions and start at 0x81200150
   // to work around this.
-  Memory::CopyToEmu(0x01200000, data.data() + 0x100, 0x700);
-  Memory::CopyToEmu(0x01300000, data.data() + 0x820, 0x1AFE00);
+  auto& memory = system.GetMemory();
+  memory.CopyToEmu(0x01200000, data.data() + 0x100, 0x700);
+  memory.CopyToEmu(0x01300000, data.data() + 0x820, 0x1AFE00);
 
-  PowerPC::ppcState.gpr[3] = 0xfff0001f;
-  PowerPC::ppcState.gpr[4] = 0x00002030;
-  PowerPC::ppcState.gpr[5] = 0x0000009c;
+  auto& ppc_state = system.GetPPCState();
+  ppc_state.gpr[3] = 0xfff0001f;
+  ppc_state.gpr[4] = 0x00002030;
+  ppc_state.gpr[5] = 0x0000009c;
 
-  MSR.FP = 1;
-  MSR.DR = 1;
-  MSR.IR = 1;
+  ppc_state.msr.FP = 1;
+  ppc_state.msr.DR = 1;
+  ppc_state.msr.IR = 1;
 
-  PowerPC::ppcState.spr[SPR_HID0] = 0x0011c464;
-  PowerPC::ppcState.spr[SPR_IBAT3U] = 0xfff0001f;
-  PowerPC::ppcState.spr[SPR_IBAT3L] = 0xfff00001;
-  PowerPC::ppcState.spr[SPR_DBAT3U] = 0xfff0001f;
-  PowerPC::ppcState.spr[SPR_DBAT3L] = 0xfff00001;
-  SetupBAT(/*is_wii*/ false);
+  ppc_state.spr[SPR_HID0] = 0x0011c464;
+  ppc_state.spr[SPR_IBAT3U] = 0xfff0001f;
+  ppc_state.spr[SPR_IBAT3L] = 0xfff00001;
+  ppc_state.spr[SPR_DBAT3U] = 0xfff0001f;
+  ppc_state.spr[SPR_DBAT3L] = 0xfff00001;
+  SetupBAT(system, /*is_wii*/ false);
 
-  PC = 0x81200150;
+  ppc_state.pc = 0x81200150;
   return true;
 }
 
@@ -484,20 +470,22 @@ static void SetDefaultDisc()
     SetDisc(DiscIO::CreateDisc(default_iso));
 }
 
-static void CopyDefaultExceptionHandlers()
+static void CopyDefaultExceptionHandlers(Core::System& system)
 {
   constexpr u32 EXCEPTION_HANDLER_ADDRESSES[] = {0x00000100, 0x00000200, 0x00000300, 0x00000400,
                                                  0x00000500, 0x00000600, 0x00000700, 0x00000800,
                                                  0x00000900, 0x00000C00, 0x00000D00, 0x00000F00,
                                                  0x00001300, 0x00001400, 0x00001700};
 
+  auto& memory = system.GetMemory();
   constexpr u32 RFI_INSTRUCTION = 0x4C000064;
   for (const u32 address : EXCEPTION_HANDLER_ADDRESSES)
-    Memory::Write_U32(RFI_INSTRUCTION, address);
+    memory.Write_U32(RFI_INSTRUCTION, address);
 }
 
 // Third boot step after BootManager and Core. See Call schedule in BootManager.cpp
-bool CBoot::BootUp(std::unique_ptr<BootParameters> boot)
+bool CBoot::BootUp(Core::System& system, const Core::CPUThreadGuard& guard,
+                   std::unique_ptr<BootParameters> boot)
 {
   SConfig& config = SConfig::GetInstance();
 
@@ -508,13 +496,15 @@ bool CBoot::BootUp(std::unique_ptr<BootParameters> boot)
   }
 
   // PAL Wii uses NTSC framerate and linecount in 60Hz modes
-  VideoInterface::Preset(DiscIO::IsNTSC(config.m_region) ||
-                         (config.bWii && Config::Get(Config::SYSCONF_PAL60)));
+  system.GetVideoInterface().Preset(DiscIO::IsNTSC(config.m_region) ||
+                                    (config.bWii && Config::Get(Config::SYSCONF_PAL60)));
 
   struct BootTitle
   {
-    BootTitle(const std::vector<DiscIO::Riivolution::Patch>& patches)
-        : config(SConfig::GetInstance()), riivolution_patches(patches)
+    BootTitle(Core::System& system_, const Core::CPUThreadGuard& guard_,
+              const std::vector<DiscIO::Riivolution::Patch>& patches)
+        : system(system_), guard(guard_), config(SConfig::GetInstance()),
+          riivolution_patches(patches)
     {
     }
     bool operator()(BootParameters::Disc& disc) const
@@ -526,10 +516,10 @@ bool CBoot::BootUp(std::unique_ptr<BootParameters> boot)
       if (!volume)
         return false;
 
-      if (!EmulatedBS2(config.bWii, *volume, riivolution_patches))
+      if (!EmulatedBS2(system, guard, config.bWii, *volume, riivolution_patches))
         return false;
 
-      SConfig::OnNewTitleLoad();
+      SConfig::OnNewTitleLoad(guard);
       return true;
     }
 
@@ -542,41 +532,43 @@ bool CBoot::BootUp(std::unique_ptr<BootParameters> boot)
 
       SetDefaultDisc();
 
-      SetupMSR();
-      SetupHID(config.bWii);
-      SetupBAT(config.bWii);
-      CopyDefaultExceptionHandlers();
+      auto& ppc_state = system.GetPPCState();
+
+      SetupMSR(ppc_state);
+      SetupHID(ppc_state, config.bWii);
+      SetupBAT(system, config.bWii);
+      CopyDefaultExceptionHandlers(system);
 
       if (config.bWii)
       {
         // Set a value for the SP. It doesn't matter where this points to,
         // as long as it is a valid location. This value is taken from a homebrew binary.
-        PowerPC::ppcState.gpr[1] = 0x8004d4bc;
+        ppc_state.gpr[1] = 0x8004d4bc;
 
         // Because there is no TMD to get the requested system (IOS) version from,
         // we default to IOS58, which is the version used by the Homebrew Channel.
-        SetupWiiMemory(IOS::HLE::IOSC::ConsoleType::Retail);
-        IOS::HLE::GetIOS()->BootIOS(Titles::IOS(58));
+        SetupWiiMemory(system, IOS::HLE::IOSC::ConsoleType::Retail);
+        IOS::HLE::GetIOS()->BootIOS(system, Titles::IOS(58));
       }
       else
       {
-        SetupGCMemory();
+        SetupGCMemory(system, guard);
       }
 
-      if (!executable.reader->LoadIntoMemory())
+      if (!executable.reader->LoadIntoMemory(system))
       {
         PanicAlertFmtT("Failed to load the executable to memory.");
         return false;
       }
 
-      SConfig::OnNewTitleLoad();
+      SConfig::OnNewTitleLoad(guard);
 
-      PC = executable.reader->GetEntryPoint();
+      ppc_state.pc = executable.reader->GetEntryPoint();
 
-      if (executable.reader->LoadSymbols())
+      if (executable.reader->LoadSymbols(guard))
       {
         UpdateDebugger_MapLoaded();
-        HLE::PatchFunctions();
+        HLE::PatchFunctions(system);
       }
       return true;
     }
@@ -584,20 +576,20 @@ bool CBoot::BootUp(std::unique_ptr<BootParameters> boot)
     bool operator()(const DiscIO::VolumeWAD& wad) const
     {
       SetDefaultDisc();
-      if (!Boot_WiiWAD(wad))
+      if (!Boot_WiiWAD(system, wad))
         return false;
 
-      SConfig::OnNewTitleLoad();
+      SConfig::OnNewTitleLoad(guard);
       return true;
     }
 
     bool operator()(const BootParameters::NANDTitle& nand_title) const
     {
       SetDefaultDisc();
-      if (!BootNANDTitle(nand_title.id))
+      if (!BootNANDTitle(system, nand_title.id))
         return false;
 
-      SConfig::OnNewTitleLoad();
+      SConfig::OnNewTitleLoad(guard);
       return true;
     }
 
@@ -613,7 +605,7 @@ bool CBoot::BootUp(std::unique_ptr<BootParameters> boot)
         return false;
       }
 
-      if (!Load_BS2(ipl.path))
+      if (!Load_BS2(system, ipl.path))
         return false;
 
       if (ipl.disc)
@@ -622,7 +614,7 @@ bool CBoot::BootUp(std::unique_ptr<BootParameters> boot)
         SetDisc(DiscIO::CreateDisc(ipl.disc->path), ipl.disc->auto_disc_change_paths);
       }
 
-      SConfig::OnNewTitleLoad();
+      SConfig::OnNewTitleLoad(guard);
       return true;
     }
 
@@ -633,14 +625,16 @@ bool CBoot::BootUp(std::unique_ptr<BootParameters> boot)
     }
 
   private:
+    Core::System& system;
+    const Core::CPUThreadGuard& guard;
     const SConfig& config;
     const std::vector<DiscIO::Riivolution::Patch>& riivolution_patches;
   };
 
-  if (!std::visit(BootTitle(boot->riivolution_patches), boot->parameters))
+  if (!std::visit(BootTitle(system, guard, boot->riivolution_patches), boot->parameters))
     return false;
 
-  DiscIO::Riivolution::ApplyGeneralMemoryPatches(boot->riivolution_patches);
+  DiscIO::Riivolution::ApplyGeneralMemoryPatches(guard, boot->riivolution_patches);
 
   return true;
 }
