@@ -15,7 +15,7 @@
 #include "Core/WiiUtils.h"
 #include "DiscIO/VolumeWad.h"
 
-bool CBoot::BootNANDTitle(const u64 title_id)
+bool CBoot::BootNANDTitle(Core::System& system, const u64 title_id)
 {
   UpdateStateFlags([](StateFlags* state) {
     state->type = 0x04;  // TYPE_NANDBOOT
@@ -28,16 +28,16 @@ bool CBoot::BootNANDTitle(const u64 title_id)
     console_type = ticket.GetConsoleType();
   else
     ERROR_LOG_FMT(BOOT, "No ticket was found for {:016x}", title_id);
-  SetupWiiMemory(console_type);
+  SetupWiiMemory(system, console_type);
   return es->LaunchTitle(title_id);
 }
 
-bool CBoot::Boot_WiiWAD(const DiscIO::VolumeWAD& wad)
+bool CBoot::Boot_WiiWAD(Core::System& system, const DiscIO::VolumeWAD& wad)
 {
   if (!WiiUtils::InstallWAD(*IOS::HLE::GetIOS(), wad, WiiUtils::InstallType::Temporary))
   {
     PanicAlertFmtT("Cannot boot this WAD because it could not be installed to the NAND.");
     return false;
   }
-  return BootNANDTitle(wad.GetTMD().GetTitleId());
+  return BootNANDTitle(system, wad.GetTMD().GetTitleId());
 }
