@@ -3,7 +3,6 @@
 
 #include "Core/CheatSearch.h"
 
-#include <cassert>
 #include <functional>
 #include <memory>
 #include <optional>
@@ -13,6 +12,7 @@
 #include <vector>
 
 #include "Common/Align.h"
+#include "Common/Assert.h"
 #include "Common/BitUtils.h"
 #include "Common/StringUtil.h"
 
@@ -20,6 +20,7 @@
 #include "Core/HW/Memmap.h"
 #include "Core/PowerPC/MMU.h"
 #include "Core/PowerPC/PowerPC.h"
+#include "Core/System.h"
 
 Cheats::DataType Cheats::GetDataType(const Cheats::SearchValue& value)
 {
@@ -93,7 +94,7 @@ std::vector<u8> Cheats::GetValueAsByteVector(const Cheats::SearchValue& value)
   case Cheats::DataType::F64:
     return ToByteVector(Common::swap64(Common::BitCast<u64>(std::get<double>(value.m_value))));
   default:
-    assert(0);
+    DEBUG_ASSERT(false);
     return {};
   }
 }
@@ -102,41 +103,47 @@ namespace
 {
 template <typename T>
 static std::optional<PowerPC::ReadResult<T>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space);
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space);
 
 template <>
 std::optional<PowerPC::ReadResult<u8>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space)
 {
-  return PowerPC::HostTryReadU8(addr, space);
+  return PowerPC::MMU::HostTryReadU8(guard, addr, space);
 }
 
 template <>
 std::optional<PowerPC::ReadResult<u16>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space)
 {
-  return PowerPC::HostTryReadU16(addr, space);
+  return PowerPC::MMU::HostTryReadU16(guard, addr, space);
 }
 
 template <>
 std::optional<PowerPC::ReadResult<u32>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space)
 {
-  return PowerPC::HostTryReadU32(addr, space);
+  return PowerPC::MMU::HostTryReadU32(guard, addr, space);
 }
 
 template <>
 std::optional<PowerPC::ReadResult<u64>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space)
 {
-  return PowerPC::HostTryReadU64(addr, space);
+  return PowerPC::MMU::HostTryReadU64(guard, addr, space);
 }
 
 template <>
 std::optional<PowerPC::ReadResult<s8>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space)
 {
-  auto tmp = PowerPC::HostTryReadU8(addr, space);
+  auto tmp = PowerPC::MMU::HostTryReadU8(guard, addr, space);
   if (!tmp)
     return std::nullopt;
   return PowerPC::ReadResult<s8>(tmp->translated, Common::BitCast<s8>(tmp->value));
@@ -144,9 +151,10 @@ TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
 
 template <>
 std::optional<PowerPC::ReadResult<s16>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space)
 {
-  auto tmp = PowerPC::HostTryReadU16(addr, space);
+  auto tmp = PowerPC::MMU::HostTryReadU16(guard, addr, space);
   if (!tmp)
     return std::nullopt;
   return PowerPC::ReadResult<s16>(tmp->translated, Common::BitCast<s16>(tmp->value));
@@ -154,9 +162,10 @@ TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
 
 template <>
 std::optional<PowerPC::ReadResult<s32>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space)
 {
-  auto tmp = PowerPC::HostTryReadU32(addr, space);
+  auto tmp = PowerPC::MMU::HostTryReadU32(guard, addr, space);
   if (!tmp)
     return std::nullopt;
   return PowerPC::ReadResult<s32>(tmp->translated, Common::BitCast<s32>(tmp->value));
@@ -164,9 +173,10 @@ TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
 
 template <>
 std::optional<PowerPC::ReadResult<s64>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space)
 {
-  auto tmp = PowerPC::HostTryReadU64(addr, space);
+  auto tmp = PowerPC::MMU::HostTryReadU64(guard, addr, space);
   if (!tmp)
     return std::nullopt;
   return PowerPC::ReadResult<s64>(tmp->translated, Common::BitCast<s64>(tmp->value));
@@ -174,22 +184,25 @@ TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
 
 template <>
 std::optional<PowerPC::ReadResult<float>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space)
 {
-  return PowerPC::HostTryReadF32(addr, space);
+  return PowerPC::MMU::HostTryReadF32(guard, addr, space);
 }
 
 template <>
 std::optional<PowerPC::ReadResult<double>>
-TryReadValueFromEmulatedMemory(u32 addr, PowerPC::RequestedAddressSpace space)
+TryReadValueFromEmulatedMemory(const Core::CPUThreadGuard& guard, u32 addr,
+                               PowerPC::RequestedAddressSpace space)
 {
-  return PowerPC::HostTryReadF64(addr, space);
+  return PowerPC::MMU::HostTryReadF64(guard, addr, space);
 }
 }  // namespace
 
 template <typename T>
 Common::Result<Cheats::SearchErrorCode, std::vector<Cheats::SearchResult<T>>>
-Cheats::NewSearch(const std::vector<Cheats::MemoryRange>& memory_ranges,
+Cheats::NewSearch(const Core::CPUThreadGuard& guard,
+                  const std::vector<Cheats::MemoryRange>& memory_ranges,
                   PowerPC::RequestedAddressSpace address_space, bool aligned,
                   const std::function<bool(const T& value)>& validator)
 {
@@ -204,7 +217,9 @@ Cheats::NewSearch(const std::vector<Cheats::MemoryRange>& memory_ranges,
       return;
     }
 
-    if (address_space == PowerPC::RequestedAddressSpace::Virtual && !MSR.DR)
+    auto& system = Core::System::GetInstance();
+    auto& ppc_state = system.GetPPCState();
+    if (address_space == PowerPC::RequestedAddressSpace::Virtual && !ppc_state.msr.DR)
     {
       error_code = Cheats::SearchErrorCode::VirtualAddressesCurrentlyNotAccessible;
       return;
@@ -226,7 +241,7 @@ Cheats::NewSearch(const std::vector<Cheats::MemoryRange>& memory_ranges,
       for (u64 i = 0; i < length; i += increment_per_loop)
       {
         const u32 addr = start_address + i;
-        const auto current_value = TryReadValueFromEmulatedMemory<T>(addr, address_space);
+        const auto current_value = TryReadValueFromEmulatedMemory<T>(guard, addr, address_space);
         if (!current_value)
           continue;
 
@@ -249,7 +264,8 @@ Cheats::NewSearch(const std::vector<Cheats::MemoryRange>& memory_ranges,
 
 template <typename T>
 Common::Result<Cheats::SearchErrorCode, std::vector<Cheats::SearchResult<T>>>
-Cheats::NextSearch(const std::vector<Cheats::SearchResult<T>>& previous_results,
+Cheats::NextSearch(const Core::CPUThreadGuard& guard,
+                   const std::vector<Cheats::SearchResult<T>>& previous_results,
                    PowerPC::RequestedAddressSpace address_space,
                    const std::function<bool(const T& new_value, const T& old_value)>& validator)
 {
@@ -263,7 +279,9 @@ Cheats::NextSearch(const std::vector<Cheats::SearchResult<T>>& previous_results,
       return;
     }
 
-    if (address_space == PowerPC::RequestedAddressSpace::Virtual && !MSR.DR)
+    auto& system = Core::System::GetInstance();
+    auto& ppc_state = system.GetPPCState();
+    if (address_space == PowerPC::RequestedAddressSpace::Virtual && !ppc_state.msr.DR)
     {
       error_code = Cheats::SearchErrorCode::VirtualAddressesCurrentlyNotAccessible;
       return;
@@ -272,7 +290,7 @@ Cheats::NextSearch(const std::vector<Cheats::SearchResult<T>>& previous_results,
     for (const auto& previous_result : previous_results)
     {
       const u32 addr = previous_result.m_address;
-      const auto current_value = TryReadValueFromEmulatedMemory<T>(addr, address_space);
+      const auto current_value = TryReadValueFromEmulatedMemory<T>(guard, addr, address_space);
       if (!current_value)
       {
         auto& r = results.emplace_back();
@@ -394,7 +412,7 @@ MakeCompareFunctionForSpecificValue(Cheats::CompareType op, const T& old_value)
   case Cheats::CompareType::GreaterOrEqual:
     return [&](const T& new_value) { return new_value >= old_value; };
   default:
-    assert(0);
+    DEBUG_ASSERT(false);
     return nullptr;
   }
 }
@@ -418,13 +436,13 @@ MakeCompareFunctionForLastValue(Cheats::CompareType op)
   case Cheats::CompareType::GreaterOrEqual:
     return [](const T& new_value, const T& old_value) { return new_value >= old_value; };
   default:
-    assert(0);
+    DEBUG_ASSERT(false);
     return nullptr;
   }
 }
 
 template <typename T>
-Cheats::SearchErrorCode Cheats::CheatSearchSession<T>::RunSearch()
+Cheats::SearchErrorCode Cheats::CheatSearchSession<T>::RunSearch(const Core::CPUThreadGuard& guard)
 {
   Common::Result<SearchErrorCode, std::vector<SearchResult<T>>> result =
       Cheats::SearchErrorCode::InvalidParameters;
@@ -437,12 +455,12 @@ Cheats::SearchErrorCode Cheats::CheatSearchSession<T>::RunSearch()
     if (m_first_search_done)
     {
       result = Cheats::NextSearch<T>(
-          m_search_results, m_address_space,
+          guard, m_search_results, m_address_space,
           [&func](const T& new_value, const T& old_value) { return func(new_value); });
     }
     else
     {
-      result = Cheats::NewSearch<T>(m_memory_ranges, m_address_space, m_aligned, func);
+      result = Cheats::NewSearch<T>(guard, m_memory_ranges, m_address_space, m_aligned, func);
     }
   }
   else if (m_filter_type == FilterType::CompareAgainstLastValue)
@@ -450,19 +468,19 @@ Cheats::SearchErrorCode Cheats::CheatSearchSession<T>::RunSearch()
     if (!m_first_search_done)
       return Cheats::SearchErrorCode::InvalidParameters;
 
-    result = Cheats::NextSearch<T>(m_search_results, m_address_space,
+    result = Cheats::NextSearch<T>(guard, m_search_results, m_address_space,
                                    MakeCompareFunctionForLastValue<T>(m_compare_type));
   }
   else if (m_filter_type == FilterType::DoNotFilter)
   {
     if (m_first_search_done)
     {
-      result = Cheats::NextSearch<T>(m_search_results, m_address_space,
+      result = Cheats::NextSearch<T>(guard, m_search_results, m_address_space,
                                      [](const T& v1, const T& v2) { return true; });
     }
     else
     {
-      result = Cheats::NewSearch<T>(m_memory_ranges, m_address_space, m_aligned,
+      result = Cheats::NewSearch<T>(guard, m_memory_ranges, m_address_space, m_aligned,
                                     [](const T& v) { return true; });
     }
   }
@@ -662,7 +680,7 @@ Cheats::MakeSession(std::vector<MemoryRange> memory_ranges,
     return std::make_unique<CheatSearchSession<double>>(std::move(memory_ranges), address_space,
                                                         aligned);
   default:
-    assert(0);
+    DEBUG_ASSERT(false);
     return nullptr;
   }
 }
