@@ -15,6 +15,7 @@
 #include "Core/HW/DSPHLE/DSPHLE.h"
 #include "Core/HW/DSPHLE/MailHandler.h"
 #include "Core/HW/DSPHLE/UCodes/UCodes.h"
+#include "Core/System.h"
 
 namespace DSP::HLE
 {
@@ -54,9 +55,15 @@ constexpr u32 FLAGS_SAMPLE_FORMAT_BYTES_SHIFT = 16;
 
 constexpr u32 SAMPLE_RATE = 48000;
 
+bool ASndUCode::SwapLeftRight() const
+{
+  return m_crc == HASH_DESERT_BUS_2011 || m_crc == HASH_DESERT_BUS_2012;
+}
+
 bool ASndUCode::UseNewFlagMasks() const
 {
-  return m_crc == HASH_2011 || m_crc == HASH_2020 || m_crc == HASH_2020_PAD;
+  return m_crc == HASH_2011 || m_crc == HASH_2020 || m_crc == HASH_2020_PAD ||
+         m_crc == HASH_DESERT_BUS_2011 || m_crc == HASH_DESERT_BUS_2012;
 }
 
 ASndUCode::ASndUCode(DSPHLE* dsphle, u32 crc) : UCodeInterface(dsphle, crc)
@@ -73,7 +80,7 @@ void ASndUCode::Update()
   // This is dubious in general, since we set the interrupt parameter on m_mail_handler.PushMail
   if (m_mail_handler.HasPending())
   {
-    DSP::GenerateDSPInterruptFromDSPEmu(DSP::INT_DSP);
+    Core::System::GetInstance().GetDSP().GenerateDSPInterruptFromDSPEmu(DSP::INT_DSP);
   }
 }
 
@@ -383,7 +390,13 @@ void ASndUCode::DoMixing(u32 return_mail)
         }
         // Both paths jmpr $AR3, which is an index into sample_selector
 
-        const auto [new_r, new_l] = (this->*sample_function)();
+        auto [new_r, new_l] = (this->*sample_function)();
+        if (SwapLeftRight())
+        {
+          // The Desert Bus versions swapped the left and right input channels so that left
+          // comes first, and then right. Before, right came before left.
+          std::swap(new_r, new_l);
+        }
         // out_samp: "multiply sample x volume" - left is put in $ax0.h, right is put in $ax1.h
 
         // All functions jumped to from sample_selector jump or fall through here (zero_samples also
