@@ -19,6 +19,7 @@
 #include "Core/IOS/ES/Formats.h"
 #include "Core/IOS/FS/FileSystem.h"
 #include "Core/IOS/Uids.h"
+#include "Core/System.h"
 
 namespace IOS::HLE
 {
@@ -97,10 +98,13 @@ IPCReply ESDevice::ImportTicket(const IOCtlVRequest& request)
   if (!request.HasNumberOfValidVectors(3, 0))
     return IPCReply(ES_EINVAL);
 
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
   std::vector<u8> bytes(request.in_vectors[0].size);
-  Memory::CopyFromEmu(bytes.data(), request.in_vectors[0].address, request.in_vectors[0].size);
+  memory.CopyFromEmu(bytes.data(), request.in_vectors[0].address, request.in_vectors[0].size);
   std::vector<u8> cert_chain(request.in_vectors[1].size);
-  Memory::CopyFromEmu(cert_chain.data(), request.in_vectors[1].address, request.in_vectors[1].size);
+  memory.CopyFromEmu(cert_chain.data(), request.in_vectors[1].address, request.in_vectors[1].size);
   return IPCReply(ImportTicket(bytes, cert_chain));
 }
 
@@ -187,8 +191,11 @@ IPCReply ESDevice::ImportTmd(Context& context, const IOCtlVRequest& request)
   if (!ES::IsValidTMDSize(request.in_vectors[0].size))
     return IPCReply(ES_EINVAL);
 
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
   std::vector<u8> tmd(request.in_vectors[0].size);
-  Memory::CopyFromEmu(tmd.data(), request.in_vectors[0].address, request.in_vectors[0].size);
+  memory.CopyFromEmu(tmd.data(), request.in_vectors[0].address, request.in_vectors[0].size);
   return IPCReply(ImportTmd(context, tmd, m_title_context.tmd.GetTitleId(),
                             m_title_context.tmd.GetTitleFlags()));
 }
@@ -273,10 +280,13 @@ IPCReply ESDevice::ImportTitleInit(Context& context, const IOCtlVRequest& reques
   if (!ES::IsValidTMDSize(request.in_vectors[0].size))
     return IPCReply(ES_EINVAL);
 
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
   std::vector<u8> tmd(request.in_vectors[0].size);
-  Memory::CopyFromEmu(tmd.data(), request.in_vectors[0].address, request.in_vectors[0].size);
+  memory.CopyFromEmu(tmd.data(), request.in_vectors[0].address, request.in_vectors[0].size);
   std::vector<u8> certs(request.in_vectors[1].size);
-  Memory::CopyFromEmu(certs.data(), request.in_vectors[1].address, request.in_vectors[1].size);
+  memory.CopyFromEmu(certs.data(), request.in_vectors[1].address, request.in_vectors[1].size);
   return IPCReply(ImportTitleInit(context, tmd, certs));
 }
 
@@ -328,8 +338,11 @@ IPCReply ESDevice::ImportContentBegin(Context& context, const IOCtlVRequest& req
   if (!request.HasNumberOfValidVectors(2, 0))
     return IPCReply(ES_EINVAL);
 
-  u64 title_id = Memory::Read_U64(request.in_vectors[0].address);
-  u32 content_id = Memory::Read_U32(request.in_vectors[1].address);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
+  u64 title_id = memory.Read_U64(request.in_vectors[0].address);
+  u32 content_id = memory.Read_U32(request.in_vectors[1].address);
   return IPCReply(ImportContentBegin(context, title_id, content_id));
 }
 
@@ -347,8 +360,11 @@ IPCReply ESDevice::ImportContentData(Context& context, const IOCtlVRequest& requ
   if (!request.HasNumberOfValidVectors(2, 0))
     return IPCReply(ES_EINVAL);
 
-  u32 content_fd = Memory::Read_U32(request.in_vectors[0].address);
-  u8* data_start = Memory::GetPointer(request.in_vectors[1].address);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
+  u32 content_fd = memory.Read_U32(request.in_vectors[0].address);
+  u8* data_start = memory.GetPointer(request.in_vectors[1].address);
   return IPCReply(ImportContentData(context, content_fd, data_start, request.in_vectors[1].size));
 }
 
@@ -430,7 +446,10 @@ IPCReply ESDevice::ImportContentEnd(Context& context, const IOCtlVRequest& reque
   if (!request.HasNumberOfValidVectors(1, 0))
     return IPCReply(ES_EINVAL);
 
-  u32 content_fd = Memory::Read_U32(request.in_vectors[0].address);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
+  u32 content_fd = memory.Read_U32(request.in_vectors[0].address);
   return IPCReply(ImportContentEnd(context, content_fd));
 }
 
@@ -542,7 +561,10 @@ IPCReply ESDevice::DeleteTitle(const IOCtlVRequest& request)
   if (!request.HasNumberOfValidVectors(1, 0) || request.in_vectors[0].size != 8)
     return IPCReply(ES_EINVAL);
 
-  const u64 title_id = Memory::Read_U64(request.in_vectors[0].address);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
+  const u64 title_id = memory.Read_U64(request.in_vectors[0].address);
   return IPCReply(DeleteTitle(title_id));
 }
 
@@ -558,15 +580,19 @@ ReturnCode ESDevice::DeleteTicket(const u8* ticket_view)
   if (!ticket.IsValid())
     return FS_ENOENT;
 
+  const bool was_v1_ticket = ticket.IsV1Ticket();
+  const std::string ticket_path =
+      was_v1_ticket ? Common::GetV1TicketFileName(title_id) : Common::GetTicketFileName(title_id);
+
   const u64 ticket_id = Common::swap64(ticket_view + offsetof(ES::TicketView, ticket_id));
   ticket.DeleteTicket(ticket_id);
 
   const std::vector<u8>& new_ticket = ticket.GetBytes();
-  const std::string ticket_path = ticket.IsV1Ticket() ? Common::GetV1TicketFileName(title_id) :
-                                                        Common::GetTicketFileName(title_id);
 
   if (!new_ticket.empty())
   {
+    ASSERT(ticket.IsValid());
+    ASSERT(ticket.IsV1Ticket() == was_v1_ticket);
     const auto file = fs->OpenFile(PID_KERNEL, PID_KERNEL, ticket_path, FS::Mode::ReadWrite);
     if (!file || !file->Write(new_ticket.data(), new_ticket.size()))
       return ES_EIO;
@@ -595,7 +621,10 @@ IPCReply ESDevice::DeleteTicket(const IOCtlVRequest& request)
   {
     return IPCReply(ES_EINVAL);
   }
-  return IPCReply(DeleteTicket(Memory::GetPointer(request.in_vectors[0].address)));
+
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+  return IPCReply(DeleteTicket(memory.GetPointer(request.in_vectors[0].address)));
 }
 
 ReturnCode ESDevice::DeleteTitleContent(u64 title_id) const
@@ -621,7 +650,10 @@ IPCReply ESDevice::DeleteTitleContent(const IOCtlVRequest& request)
 {
   if (!request.HasNumberOfValidVectors(1, 0) || request.in_vectors[0].size != sizeof(u64))
     return IPCReply(ES_EINVAL);
-  return IPCReply(DeleteTitleContent(Memory::Read_U64(request.in_vectors[0].address)));
+
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+  return IPCReply(DeleteTitleContent(memory.Read_U64(request.in_vectors[0].address)));
 }
 
 ReturnCode ESDevice::DeleteContent(u64 title_id, u32 content_id) const
@@ -649,8 +681,11 @@ IPCReply ESDevice::DeleteContent(const IOCtlVRequest& request)
   {
     return IPCReply(ES_EINVAL);
   }
-  return IPCReply(DeleteContent(Memory::Read_U64(request.in_vectors[0].address),
-                                Memory::Read_U32(request.in_vectors[1].address)));
+
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+  return IPCReply(DeleteContent(memory.Read_U64(request.in_vectors[0].address),
+                                memory.Read_U32(request.in_vectors[1].address)));
 }
 
 ReturnCode ESDevice::ExportTitleInit(Context& context, u64 title_id, u8* tmd_bytes, u32 tmd_size,
@@ -687,8 +722,11 @@ IPCReply ESDevice::ExportTitleInit(Context& context, const IOCtlVRequest& reques
   if (!request.HasNumberOfValidVectors(1, 1) || request.in_vectors[0].size != 8)
     return IPCReply(ES_EINVAL);
 
-  const u64 title_id = Memory::Read_U64(request.in_vectors[0].address);
-  u8* tmd_bytes = Memory::GetPointer(request.io_vectors[0].address);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
+  const u64 title_id = memory.Read_U64(request.in_vectors[0].address);
+  u8* tmd_bytes = memory.GetPointer(request.io_vectors[0].address);
   const u32 tmd_size = request.io_vectors[0].size;
 
   return IPCReply(ExportTitleInit(context, title_id, tmd_bytes, tmd_size,
@@ -733,8 +771,11 @@ IPCReply ESDevice::ExportContentBegin(Context& context, const IOCtlVRequest& req
       request.in_vectors[1].size != 4)
     return IPCReply(ES_EINVAL);
 
-  const u64 title_id = Memory::Read_U64(request.in_vectors[0].address);
-  const u32 content_id = Memory::Read_U32(request.in_vectors[1].address);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
+  const u64 title_id = memory.Read_U64(request.in_vectors[0].address);
+  const u32 content_id = memory.Read_U32(request.in_vectors[1].address);
 
   return IPCReply(ExportContentBegin(context, title_id, content_id));
 }
@@ -781,8 +822,11 @@ IPCReply ESDevice::ExportContentData(Context& context, const IOCtlVRequest& requ
     return IPCReply(ES_EINVAL);
   }
 
-  const u32 content_fd = Memory::Read_U32(request.in_vectors[0].address);
-  u8* data = Memory::GetPointer(request.io_vectors[0].address);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
+  const u32 content_fd = memory.Read_U32(request.in_vectors[0].address);
+  u8* data = memory.GetPointer(request.io_vectors[0].address);
   const u32 bytes_to_read = request.io_vectors[0].size;
 
   return IPCReply(ExportContentData(context, content_fd, data, bytes_to_read));
@@ -800,7 +844,10 @@ IPCReply ESDevice::ExportContentEnd(Context& context, const IOCtlVRequest& reque
   if (!request.HasNumberOfValidVectors(1, 0) || request.in_vectors[0].size != 4)
     return IPCReply(ES_EINVAL);
 
-  const u32 content_fd = Memory::Read_U32(request.in_vectors[0].address);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
+  const u32 content_fd = memory.Read_U32(request.in_vectors[0].address);
   return IPCReply(ExportContentEnd(context, content_fd));
 }
 
@@ -857,7 +904,11 @@ IPCReply ESDevice::DeleteSharedContent(const IOCtlVRequest& request)
   std::array<u8, 20> sha1;
   if (!request.HasNumberOfValidVectors(1, 0) || request.in_vectors[0].size != sha1.size())
     return IPCReply(ES_EINVAL);
-  Memory::CopyFromEmu(sha1.data(), request.in_vectors[0].address, request.in_vectors[0].size);
+
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
+  memory.CopyFromEmu(sha1.data(), request.in_vectors[0].address, request.in_vectors[0].size);
   return IPCReply(DeleteSharedContent(sha1));
 }
 }  // namespace IOS::HLE
