@@ -17,6 +17,7 @@
 #include "Common/CommonTypes.h"
 #include "Common/Logging/Log.h"
 #include "Common/Timer.h"
+
 #include "Core/Boot/AncastTypes.h"
 #include "Core/Boot/DolReader.h"
 #include "Core/Boot/ElfReader.h"
@@ -104,6 +105,9 @@ constexpr u32 PLACEHOLDER = 0xDEADBEEF;
 
 static bool SetupMemory(u64 ios_title_id, MemorySetupType setup_type)
 {
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
   auto target_imv = std::find_if(
       GetMemoryValues().begin(), GetMemoryValues().end(),
       [&](const MemoryValues& imv) { return imv.ios_number == (ios_title_id & 0xffff); });
@@ -116,7 +120,7 @@ static bool SetupMemory(u64 ios_title_id, MemorySetupType setup_type)
 
   if (setup_type == MemorySetupType::IOSReload)
   {
-    Memory::Write_U32(target_imv->ios_version, ADDR_IOS_VERSION);
+    memory.Write_U32(target_imv->ios_version, ADDR_IOS_VERSION);
 
     // These values are written by the IOS kernel as part of its boot process (for IOS28 and newer).
     //
@@ -127,15 +131,15 @@ static bool SetupMemory(u64 ios_title_id, MemorySetupType setup_type)
     // the new IOS either updates the range (>= IOS28) or inherits it (< IOS28).
     //
     // We can skip this convoluted process and just write the correct range directly.
-    Memory::Write_U32(target_imv->mem2_physical_size, ADDR_MEM2_SIZE);
-    Memory::Write_U32(target_imv->mem2_simulated_size, ADDR_MEM2_SIM_SIZE);
-    Memory::Write_U32(target_imv->mem2_end, ADDR_MEM2_END);
-    Memory::Write_U32(target_imv->mem2_arena_begin, ADDR_MEM2_ARENA_BEGIN);
-    Memory::Write_U32(target_imv->mem2_arena_end, ADDR_MEM2_ARENA_END);
-    Memory::Write_U32(target_imv->ipc_buffer_begin, ADDR_IPC_BUFFER_BEGIN);
-    Memory::Write_U32(target_imv->ipc_buffer_end, ADDR_IPC_BUFFER_END);
-    Memory::Write_U32(target_imv->ios_reserved_begin, ADDR_IOS_RESERVED_BEGIN);
-    Memory::Write_U32(target_imv->ios_reserved_end, ADDR_IOS_RESERVED_END);
+    memory.Write_U32(target_imv->mem2_physical_size, ADDR_MEM2_SIZE);
+    memory.Write_U32(target_imv->mem2_simulated_size, ADDR_MEM2_SIM_SIZE);
+    memory.Write_U32(target_imv->mem2_end, ADDR_MEM2_END);
+    memory.Write_U32(target_imv->mem2_arena_begin, ADDR_MEM2_ARENA_BEGIN);
+    memory.Write_U32(target_imv->mem2_arena_end, ADDR_MEM2_ARENA_END);
+    memory.Write_U32(target_imv->ipc_buffer_begin, ADDR_IPC_BUFFER_BEGIN);
+    memory.Write_U32(target_imv->ipc_buffer_end, ADDR_IPC_BUFFER_END);
+    memory.Write_U32(target_imv->ios_reserved_begin, ADDR_IOS_RESERVED_BEGIN);
+    memory.Write_U32(target_imv->ios_reserved_end, ADDR_IOS_RESERVED_END);
 
     RAMOverrideForIOSMemoryValues(setup_type);
 
@@ -146,40 +150,40 @@ static bool SetupMemory(u64 ios_title_id, MemorySetupType setup_type)
   // and system information (see below).
   constexpr u32 LOW_MEM1_REGION_START = 0;
   constexpr u32 LOW_MEM1_REGION_SIZE = 0x3fff;
-  Memory::Memset(LOW_MEM1_REGION_START, 0, LOW_MEM1_REGION_SIZE);
+  memory.Memset(LOW_MEM1_REGION_START, 0, LOW_MEM1_REGION_SIZE);
 
-  Memory::Write_U32(target_imv->mem1_physical_size, ADDR_MEM1_SIZE);
-  Memory::Write_U32(target_imv->mem1_simulated_size, ADDR_MEM1_SIM_SIZE);
-  Memory::Write_U32(target_imv->mem1_end, ADDR_MEM1_END);
-  Memory::Write_U32(target_imv->mem1_arena_begin, ADDR_MEM1_ARENA_BEGIN);
-  Memory::Write_U32(target_imv->mem1_arena_end, ADDR_MEM1_ARENA_END);
-  Memory::Write_U32(PLACEHOLDER, ADDR_PH1);
-  Memory::Write_U32(target_imv->mem2_physical_size, ADDR_MEM2_SIZE);
-  Memory::Write_U32(target_imv->mem2_simulated_size, ADDR_MEM2_SIM_SIZE);
-  Memory::Write_U32(target_imv->mem2_end, ADDR_MEM2_END);
-  Memory::Write_U32(target_imv->mem2_arena_begin, ADDR_MEM2_ARENA_BEGIN);
-  Memory::Write_U32(target_imv->mem2_arena_end, ADDR_MEM2_ARENA_END);
-  Memory::Write_U32(PLACEHOLDER, ADDR_PH2);
-  Memory::Write_U32(target_imv->ipc_buffer_begin, ADDR_IPC_BUFFER_BEGIN);
-  Memory::Write_U32(target_imv->ipc_buffer_end, ADDR_IPC_BUFFER_END);
-  Memory::Write_U32(target_imv->hollywood_revision, ADDR_HOLLYWOOD_REVISION);
-  Memory::Write_U32(PLACEHOLDER, ADDR_PH3);
-  Memory::Write_U32(target_imv->ios_version, ADDR_IOS_VERSION);
-  Memory::Write_U32(target_imv->ios_date, ADDR_IOS_DATE);
-  Memory::Write_U32(target_imv->ios_reserved_begin, ADDR_IOS_RESERVED_BEGIN);
-  Memory::Write_U32(target_imv->ios_reserved_end, ADDR_IOS_RESERVED_END);
-  Memory::Write_U32(PLACEHOLDER, ADDR_PH4);
-  Memory::Write_U32(PLACEHOLDER, ADDR_PH5);
-  Memory::Write_U32(target_imv->ram_vendor, ADDR_RAM_VENDOR);
-  Memory::Write_U8(0xDE, ADDR_BOOT_FLAG);
-  Memory::Write_U8(0xAD, ADDR_APPLOADER_FLAG);
-  Memory::Write_U16(0xBEEF, ADDR_DEVKIT_BOOT_PROGRAM_VERSION);
-  Memory::Write_U32(target_imv->sysmenu_sync, ADDR_SYSMENU_SYNC);
+  memory.Write_U32(target_imv->mem1_physical_size, ADDR_MEM1_SIZE);
+  memory.Write_U32(target_imv->mem1_simulated_size, ADDR_MEM1_SIM_SIZE);
+  memory.Write_U32(target_imv->mem1_end, ADDR_MEM1_END);
+  memory.Write_U32(target_imv->mem1_arena_begin, ADDR_MEM1_ARENA_BEGIN);
+  memory.Write_U32(target_imv->mem1_arena_end, ADDR_MEM1_ARENA_END);
+  memory.Write_U32(PLACEHOLDER, ADDR_PH1);
+  memory.Write_U32(target_imv->mem2_physical_size, ADDR_MEM2_SIZE);
+  memory.Write_U32(target_imv->mem2_simulated_size, ADDR_MEM2_SIM_SIZE);
+  memory.Write_U32(target_imv->mem2_end, ADDR_MEM2_END);
+  memory.Write_U32(target_imv->mem2_arena_begin, ADDR_MEM2_ARENA_BEGIN);
+  memory.Write_U32(target_imv->mem2_arena_end, ADDR_MEM2_ARENA_END);
+  memory.Write_U32(PLACEHOLDER, ADDR_PH2);
+  memory.Write_U32(target_imv->ipc_buffer_begin, ADDR_IPC_BUFFER_BEGIN);
+  memory.Write_U32(target_imv->ipc_buffer_end, ADDR_IPC_BUFFER_END);
+  memory.Write_U32(target_imv->hollywood_revision, ADDR_HOLLYWOOD_REVISION);
+  memory.Write_U32(PLACEHOLDER, ADDR_PH3);
+  memory.Write_U32(target_imv->ios_version, ADDR_IOS_VERSION);
+  memory.Write_U32(target_imv->ios_date, ADDR_IOS_DATE);
+  memory.Write_U32(target_imv->ios_reserved_begin, ADDR_IOS_RESERVED_BEGIN);
+  memory.Write_U32(target_imv->ios_reserved_end, ADDR_IOS_RESERVED_END);
+  memory.Write_U32(PLACEHOLDER, ADDR_PH4);
+  memory.Write_U32(PLACEHOLDER, ADDR_PH5);
+  memory.Write_U32(target_imv->ram_vendor, ADDR_RAM_VENDOR);
+  memory.Write_U8(0xDE, ADDR_BOOT_FLAG);
+  memory.Write_U8(0xAD, ADDR_APPLOADER_FLAG);
+  memory.Write_U16(0xBEEF, ADDR_DEVKIT_BOOT_PROGRAM_VERSION);
+  memory.Write_U32(target_imv->sysmenu_sync, ADDR_SYSMENU_SYNC);
 
-  Memory::Write_U32(target_imv->mem1_physical_size, ADDR_LEGACY_MEM_SIZE);
-  Memory::Write_U32(target_imv->mem1_arena_begin, ADDR_LEGACY_ARENA_LOW);
-  Memory::Write_U32(target_imv->mem1_arena_end, ADDR_LEGACY_ARENA_HIGH);
-  Memory::Write_U32(target_imv->mem1_simulated_size, ADDR_LEGACY_MEM_SIM_SIZE);
+  memory.Write_U32(target_imv->mem1_physical_size, ADDR_LEGACY_MEM_SIZE);
+  memory.Write_U32(target_imv->mem1_arena_begin, ADDR_LEGACY_ARENA_LOW);
+  memory.Write_U32(target_imv->mem1_arena_end, ADDR_LEGACY_ARENA_HIGH);
+  memory.Write_U32(target_imv->mem1_simulated_size, ADDR_LEGACY_MEM_SIM_SIZE);
 
   RAMOverrideForIOSMemoryValues(setup_type);
 
@@ -193,28 +197,35 @@ static bool SetupMemory(u64 ios_title_id, MemorySetupType setup_type)
 static void ResetAndPausePPC()
 {
   // This should be cleared when the PPC is released so that the write is not observable.
-  Memory::Write_U32(0x48000000, 0x00000000);  // b 0x0
-  PowerPC::Reset();
-  PC = 0;
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+  memory.Write_U32(0x48000000, 0x00000000);  // b 0x0
+  auto& power_pc = system.GetPowerPC();
+  power_pc.Reset();
+  power_pc.GetPPCState().pc = 0;
 }
 
 static void ReleasePPC()
 {
-  Memory::Write_U32(0, 0);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+  memory.Write_U32(0, 0);
   // HLE the bootstub that jumps to 0x3400.
   // NAND titles start with address translation off at 0x3400 (via the PPC bootstub)
   // The state of other CPU registers (like the BAT registers) doesn't matter much
   // because the realmode code at 0x3400 initializes everything itself anyway.
-  PC = 0x3400;
+  system.GetPPCState().pc = 0x3400;
 }
 
 static void ReleasePPCAncast()
 {
-  Memory::Write_U32(0, 0);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+  memory.Write_U32(0, 0);
   // On a real console the Espresso verifies and decrypts the Ancast image,
   // then jumps to the decrypted ancast body.
   // The Ancast loader already did this, so just jump to the decrypted body.
-  PC = ESPRESSO_ANCAST_LOCATION_VIRT + sizeof(EspressoAncastHeader);
+  system.GetPPCState().pc = ESPRESSO_ANCAST_LOCATION_VIRT + sizeof(EspressoAncastHeader);
 }
 
 void RAMOverrideForIOSMemoryValues(MemorySetupType setup_type)
@@ -223,19 +234,22 @@ void RAMOverrideForIOSMemoryValues(MemorySetupType setup_type)
   if (!Config::Get(Config::MAIN_RAM_OVERRIDE_ENABLE))
     return;
 
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+
   // Some unstated constants that can be inferred.
   const u32 ipc_buffer_size =
-      Memory::Read_U32(ADDR_IPC_BUFFER_END) - Memory::Read_U32(ADDR_IPC_BUFFER_BEGIN);
+      memory.Read_U32(ADDR_IPC_BUFFER_END) - memory.Read_U32(ADDR_IPC_BUFFER_BEGIN);
   const u32 ios_reserved_size =
-      Memory::Read_U32(ADDR_IOS_RESERVED_END) - Memory::Read_U32(ADDR_IOS_RESERVED_BEGIN);
+      memory.Read_U32(ADDR_IOS_RESERVED_END) - memory.Read_U32(ADDR_IOS_RESERVED_BEGIN);
 
-  const u32 mem1_physical_size = Memory::GetRamSizeReal();
-  const u32 mem1_simulated_size = Memory::GetRamSizeReal();
+  const u32 mem1_physical_size = memory.GetRamSizeReal();
+  const u32 mem1_simulated_size = memory.GetRamSizeReal();
   const u32 mem1_end = Memory::MEM1_BASE_ADDR + mem1_simulated_size;
   const u32 mem1_arena_begin = 0;
   const u32 mem1_arena_end = mem1_end;
-  const u32 mem2_physical_size = Memory::GetExRamSizeReal();
-  const u32 mem2_simulated_size = Memory::GetExRamSizeReal();
+  const u32 mem2_physical_size = memory.GetExRamSizeReal();
+  const u32 mem2_simulated_size = memory.GetExRamSizeReal();
   const u32 mem2_end = Memory::MEM2_BASE_ADDR + mem2_simulated_size - ios_reserved_size;
   const u32 mem2_arena_begin = Memory::MEM2_BASE_ADDR + 0x800U;
   const u32 mem2_arena_end = mem2_end - ipc_buffer_size;
@@ -247,31 +261,33 @@ void RAMOverrideForIOSMemoryValues(MemorySetupType setup_type)
   if (setup_type == MemorySetupType::Full)
   {
     // Overwriting these after the game's apploader sets them would be bad
-    Memory::Write_U32(mem1_physical_size, ADDR_MEM1_SIZE);
-    Memory::Write_U32(mem1_simulated_size, ADDR_MEM1_SIM_SIZE);
-    Memory::Write_U32(mem1_end, ADDR_MEM1_END);
-    Memory::Write_U32(mem1_arena_begin, ADDR_MEM1_ARENA_BEGIN);
-    Memory::Write_U32(mem1_arena_end, ADDR_MEM1_ARENA_END);
+    memory.Write_U32(mem1_physical_size, ADDR_MEM1_SIZE);
+    memory.Write_U32(mem1_simulated_size, ADDR_MEM1_SIM_SIZE);
+    memory.Write_U32(mem1_end, ADDR_MEM1_END);
+    memory.Write_U32(mem1_arena_begin, ADDR_MEM1_ARENA_BEGIN);
+    memory.Write_U32(mem1_arena_end, ADDR_MEM1_ARENA_END);
 
-    Memory::Write_U32(mem1_physical_size, ADDR_LEGACY_MEM_SIZE);
-    Memory::Write_U32(mem1_arena_begin, ADDR_LEGACY_ARENA_LOW);
-    Memory::Write_U32(mem1_arena_end, ADDR_LEGACY_ARENA_HIGH);
-    Memory::Write_U32(mem1_simulated_size, ADDR_LEGACY_MEM_SIM_SIZE);
+    memory.Write_U32(mem1_physical_size, ADDR_LEGACY_MEM_SIZE);
+    memory.Write_U32(mem1_arena_begin, ADDR_LEGACY_ARENA_LOW);
+    memory.Write_U32(mem1_arena_end, ADDR_LEGACY_ARENA_HIGH);
+    memory.Write_U32(mem1_simulated_size, ADDR_LEGACY_MEM_SIM_SIZE);
   }
-  Memory::Write_U32(mem2_physical_size, ADDR_MEM2_SIZE);
-  Memory::Write_U32(mem2_simulated_size, ADDR_MEM2_SIM_SIZE);
-  Memory::Write_U32(mem2_end, ADDR_MEM2_END);
-  Memory::Write_U32(mem2_arena_begin, ADDR_MEM2_ARENA_BEGIN);
-  Memory::Write_U32(mem2_arena_end, ADDR_MEM2_ARENA_END);
-  Memory::Write_U32(ipc_buffer_begin, ADDR_IPC_BUFFER_BEGIN);
-  Memory::Write_U32(ipc_buffer_end, ADDR_IPC_BUFFER_END);
-  Memory::Write_U32(ios_reserved_begin, ADDR_IOS_RESERVED_BEGIN);
-  Memory::Write_U32(ios_reserved_end, ADDR_IOS_RESERVED_END);
+  memory.Write_U32(mem2_physical_size, ADDR_MEM2_SIZE);
+  memory.Write_U32(mem2_simulated_size, ADDR_MEM2_SIM_SIZE);
+  memory.Write_U32(mem2_end, ADDR_MEM2_END);
+  memory.Write_U32(mem2_arena_begin, ADDR_MEM2_ARENA_BEGIN);
+  memory.Write_U32(mem2_arena_end, ADDR_MEM2_ARENA_END);
+  memory.Write_U32(ipc_buffer_begin, ADDR_IPC_BUFFER_BEGIN);
+  memory.Write_U32(ipc_buffer_end, ADDR_IPC_BUFFER_END);
+  memory.Write_U32(ios_reserved_begin, ADDR_IOS_RESERVED_BEGIN);
+  memory.Write_U32(ios_reserved_end, ADDR_IOS_RESERVED_END);
 }
 
 void WriteReturnValue(s32 value, u32 address)
 {
-  Memory::Write_U32(static_cast<u32>(value), address);
+  auto& system = Core::System::GetInstance();
+  auto& memory = system.GetMemory();
+  memory.Write_U32(static_cast<u32>(value), address);
 }
 
 Kernel::Kernel(IOSC::ConsoleType console_type) : m_iosc(console_type)
@@ -292,6 +308,7 @@ Kernel::~Kernel()
   {
     std::lock_guard lock(m_device_map_mutex);
     m_device_map.clear();
+    m_socket_manager.reset();
   }
 
   if (m_is_responsible_for_nand_root)
@@ -302,7 +319,8 @@ Kernel::Kernel(u64 title_id) : m_title_id(title_id)
 {
 }
 
-EmulationKernel::EmulationKernel(u64 title_id) : Kernel(title_id)
+EmulationKernel::EmulationKernel(Core::System& system, u64 title_id)
+    : Kernel(title_id), m_system(system)
 {
   INFO_LOG_FMT(IOS, "Starting IOS {:016x}", title_id);
 
@@ -347,6 +365,11 @@ std::shared_ptr<ESDevice> Kernel::GetES()
   return std::static_pointer_cast<ESDevice>(m_device_map.at("/dev/es"));
 }
 
+std::shared_ptr<WiiSockMan> EmulationKernel::GetSocketManager()
+{
+  return m_socket_manager;
+}
+
 // Since we don't have actual processes, we keep track of only the PPC's UID/GID.
 // These functions roughly correspond to syscalls 0x2b, 0x2c, 0x2d, 0x2e (though only for the PPC).
 void Kernel::SetUidForPPC(u32 uid)
@@ -388,7 +411,7 @@ static std::vector<u8> ReadBootContent(FSDevice* fs, const std::string& path, si
 
 // This corresponds to syscall 0x41, which loads a binary from the NAND and bootstraps the PPC.
 // Unlike 0x42, IOS will set up some constants in memory before booting the PPC.
-bool Kernel::BootstrapPPC(const std::string& boot_content_path)
+bool Kernel::BootstrapPPC(Core::System& system, const std::string& boot_content_path)
 {
   // Seeking and processing overhead is ignored as most time is spent reading from the NAND.
   u64 ticks = 0;
@@ -407,12 +430,11 @@ bool Kernel::BootstrapPPC(const std::string& boot_content_path)
   if (dol.IsAncast())
     INFO_LOG_FMT(IOS, "BootstrapPPC: Loading ancast image");
 
-  if (!dol.LoadIntoMemory())
+  if (!dol.LoadIntoMemory(system))
     return false;
 
   INFO_LOG_FMT(IOS, "BootstrapPPC: {}", boot_content_path);
-  Core::System::GetInstance().GetCoreTiming().ScheduleEvent(ticks, s_event_finish_ppc_bootstrap,
-                                                            dol.IsAncast());
+  system.GetCoreTiming().ScheduleEvent(ticks, s_event_finish_ppc_bootstrap, dol.IsAncast());
   return true;
 }
 
@@ -441,11 +463,11 @@ private:
   std::vector<u8> m_bytes;
 };
 
-static void FinishIOSBoot(u64 ios_title_id)
+static void FinishIOSBoot(Core::System& system, u64 ios_title_id)
 {
   // Shut down the active IOS first before switching to the new one.
   s_ios.reset();
-  s_ios = std::make_unique<EmulationKernel>(ios_title_id);
+  s_ios = std::make_unique<EmulationKernel>(system, ios_title_id);
 }
 
 static constexpr SystemTimers::TimeBaseTick GetIOSBootTicks(u32 version)
@@ -463,7 +485,8 @@ static constexpr SystemTimers::TimeBaseTick GetIOSBootTicks(u32 version)
 // Passing a boot content path is optional because we do not require IOSes
 // to be installed at the moment. If one is passed, the boot binary must exist
 // on the NAND, or the call will fail like on a Wii.
-bool Kernel::BootIOS(const u64 ios_title_id, HangPPC hang_ppc, const std::string& boot_content_path)
+bool Kernel::BootIOS(Core::System& system, const u64 ios_title_id, HangPPC hang_ppc,
+                     const std::string& boot_content_path)
 {
   // IOS suspends regular PPC<->ARM IPC before loading a new IOS.
   // IPC is not resumed if the boot fails for any reason.
@@ -479,7 +502,7 @@ bool Kernel::BootIOS(const u64 ios_title_id, HangPPC hang_ppc, const std::string
       return false;
 
     ElfReader elf{binary.GetElf()};
-    if (!elf.LoadIntoMemory(true))
+    if (!elf.LoadIntoMemory(system, true))
       return false;
   }
 
@@ -488,12 +511,12 @@ bool Kernel::BootIOS(const u64 ios_title_id, HangPPC hang_ppc, const std::string
 
   if (Core::IsRunningAndStarted())
   {
-    Core::System::GetInstance().GetCoreTiming().ScheduleEvent(
-        GetIOSBootTicks(GetVersion()), s_event_finish_ios_boot, ios_title_id);
+    system.GetCoreTiming().ScheduleEvent(GetIOSBootTicks(GetVersion()), s_event_finish_ios_boot,
+                                         ios_title_id);
   }
   else
   {
-    FinishIOSBoot(ios_title_id);
+    FinishIOSBoot(system, ios_title_id);
   }
 
   return true;
@@ -522,15 +545,17 @@ void Kernel::AddCoreDevices()
   std::lock_guard lock(m_device_map_mutex);
   AddDevice(std::make_unique<FSDevice>(*this, "/dev/fs"));
   AddDevice(std::make_unique<ESDevice>(*this, "/dev/es"));
-  AddDevice(std::make_unique<DolphinDevice>(*this, "/dev/dolphin"));
 }
 
-void Kernel::AddStaticDevices()
+void EmulationKernel::AddStaticDevices()
 {
   std::lock_guard lock(m_device_map_mutex);
 
   const Feature features = GetFeatures(GetVersion());
 
+  // Dolphin-specific device for letting homebrew access and alter emulator state.
+  AddDevice(std::make_unique<DolphinDevice>(*this, "/dev/dolphin"));
+
   // OH1 (Bluetooth)
   AddDevice(std::make_unique<DeviceStub>(*this, "/dev/usb/oh1"));
   if (!Config::Get(Config::MAIN_BLUETOOTH_PASSTHROUGH_ENABLED))
@@ -546,6 +571,11 @@ void Kernel::AddStaticDevices()
   AddDevice(std::make_unique<DeviceStub>(*this, "/dev/sdio/slot1"));
 
   // Network modules
+  if (HasFeature(features, Feature::KD) || HasFeature(features, Feature::SO) ||
+      HasFeature(features, Feature::SSL))
+  {
+    m_socket_manager = std::make_shared<IOS::HLE::WiiSockMan>();
+  }
   if (HasFeature(features, Feature::KD))
   {
     AddDevice(std::make_unique<NetKDRequestDevice>(*this, "/dev/net/kd/request"));
@@ -594,7 +624,7 @@ void Kernel::AddStaticDevices()
   }
 }
 
-s32 Kernel::GetFreeDeviceID()
+s32 EmulationKernel::GetFreeDeviceID()
 {
   for (u32 i = 0; i < IPC_MAX_FDS; i++)
   {
@@ -620,7 +650,7 @@ std::shared_ptr<Device> EmulationKernel::GetDeviceByName(std::string_view device
 }
 
 // Returns the FD for the newly opened device (on success) or an error code.
-std::optional<IPCReply> Kernel::OpenDevice(OpenRequest& request)
+std::optional<IPCReply> EmulationKernel::OpenDevice(OpenRequest& request)
 {
   const s32 new_fd = GetFreeDeviceID();
   INFO_LOG_FMT(IOS, "Opening {} (mode {}, fd {})", request.path, static_cast<u32>(request.flags),
@@ -662,14 +692,14 @@ std::optional<IPCReply> Kernel::OpenDevice(OpenRequest& request)
   return result;
 }
 
-std::optional<IPCReply> Kernel::HandleIPCCommand(const Request& request)
+std::optional<IPCReply> EmulationKernel::HandleIPCCommand(const Request& request)
 {
   if (request.command < IPC_CMD_OPEN || request.command > IPC_CMD_IOCTLV)
     return IPCReply{IPC_EINVAL, 978_tbticks};
 
   if (request.command == IPC_CMD_OPEN)
   {
-    OpenRequest open_request{request.address};
+    OpenRequest open_request{GetSystem(), request.address};
     return OpenDevice(open_request);
   }
 
@@ -687,19 +717,19 @@ std::optional<IPCReply> Kernel::HandleIPCCommand(const Request& request)
     ret = device->Close(request.fd);
     break;
   case IPC_CMD_READ:
-    ret = device->Read(ReadWriteRequest{request.address});
+    ret = device->Read(ReadWriteRequest{GetSystem(), request.address});
     break;
   case IPC_CMD_WRITE:
-    ret = device->Write(ReadWriteRequest{request.address});
+    ret = device->Write(ReadWriteRequest{GetSystem(), request.address});
     break;
   case IPC_CMD_SEEK:
-    ret = device->Seek(SeekRequest{request.address});
+    ret = device->Seek(SeekRequest{GetSystem(), request.address});
     break;
   case IPC_CMD_IOCTL:
-    ret = device->IOCtl(IOCtlRequest{request.address});
+    ret = device->IOCtl(IOCtlRequest{GetSystem(), request.address});
     break;
   case IPC_CMD_IOCTLV:
-    ret = device->IOCtlV(IOCtlVRequest{request.address});
+    ret = device->IOCtlV(IOCtlVRequest{GetSystem(), request.address});
     break;
   default:
     ASSERT_MSG(IOS, false, "Unexpected command: {:#x}", static_cast<u32>(request.command));
@@ -718,17 +748,16 @@ std::optional<IPCReply> Kernel::HandleIPCCommand(const Request& request)
   return ret;
 }
 
-void Kernel::ExecuteIPCCommand(const u32 address)
+void EmulationKernel::ExecuteIPCCommand(const u32 address)
 {
-  Request request{address};
+  Request request{GetSystem(), address};
   std::optional<IPCReply> result = HandleIPCCommand(request);
 
   if (!result)
     return;
 
   // Ensure replies happen in order
-  auto& system = Core::System::GetInstance();
-  auto& core_timing = system.GetCoreTiming();
+  auto& core_timing = GetSystem().GetCoreTiming();
   const s64 ticks_until_last_reply = m_last_reply_time - core_timing.GetTicks();
   if (ticks_until_last_reply > 0)
     result->reply_delay_ticks += ticks_until_last_reply;
@@ -738,29 +767,30 @@ void Kernel::ExecuteIPCCommand(const u32 address)
 }
 
 // Happens AS SOON AS IPC gets a new pointer!
-void Kernel::EnqueueIPCRequest(u32 address)
+void EmulationKernel::EnqueueIPCRequest(u32 address)
 {
   // Based on hardware tests, IOS takes between 5µs and 10µs to acknowledge an IPC request.
   // Console 1: 456 TB ticks before ACK
   // Console 2: 658 TB ticks before ACK
-  Core::System::GetInstance().GetCoreTiming().ScheduleEvent(500_tbticks, s_event_enqueue,
-                                                            address | ENQUEUE_REQUEST_FLAG);
+  GetSystem().GetCoreTiming().ScheduleEvent(500_tbticks, s_event_enqueue,
+                                            address | ENQUEUE_REQUEST_FLAG);
 }
 
 // Called to send a reply to an IOS syscall
-void Kernel::EnqueueIPCReply(const Request& request, const s32 return_value, s64 cycles_in_future,
-                             CoreTiming::FromThread from)
+void EmulationKernel::EnqueueIPCReply(const Request& request, const s32 return_value,
+                                      s64 cycles_in_future, CoreTiming::FromThread from)
 {
-  Memory::Write_U32(static_cast<u32>(return_value), request.address + 4);
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+  memory.Write_U32(static_cast<u32>(return_value), request.address + 4);
   // IOS writes back the command that was responded to in the FD field.
-  Memory::Write_U32(request.command, request.address + 8);
+  memory.Write_U32(request.command, request.address + 8);
   // IOS also overwrites the command type with the reply type.
-  Memory::Write_U32(IPC_REPLY, request.address);
-  Core::System::GetInstance().GetCoreTiming().ScheduleEvent(cycles_in_future, s_event_enqueue,
-                                                            request.address, from);
+  memory.Write_U32(IPC_REPLY, request.address);
+  system.GetCoreTiming().ScheduleEvent(cycles_in_future, s_event_enqueue, request.address, from);
 }
 
-void Kernel::HandleIPCEvent(u64 userdata)
+void EmulationKernel::HandleIPCEvent(u64 userdata)
 {
   if (userdata & ENQUEUE_REQUEST_FLAG)
     m_request_queue.push_back(static_cast<u32>(userdata));
@@ -770,7 +800,7 @@ void Kernel::HandleIPCEvent(u64 userdata)
   UpdateIPC();
 }
 
-void Kernel::UpdateIPC()
+void EmulationKernel::UpdateIPC()
 {
   if (m_ipc_paused || !IsReady())
     return;
@@ -794,7 +824,7 @@ void Kernel::UpdateIPC()
   }
 }
 
-void Kernel::UpdateDevices()
+void EmulationKernel::UpdateDevices()
 {
   // Check if a hardware device must be updated
   for (const auto& entry : m_device_map)
@@ -806,14 +836,15 @@ void Kernel::UpdateDevices()
   }
 }
 
-void Kernel::UpdateWantDeterminism(const bool new_want_determinism)
+void EmulationKernel::UpdateWantDeterminism(const bool new_want_determinism)
 {
-  WiiSockMan::GetInstance().UpdateWantDeterminism(new_want_determinism);
+  if (m_socket_manager)
+    m_socket_manager->UpdateWantDeterminism(new_want_determinism);
   for (const auto& device : m_device_map)
     device.second->UpdateWantDeterminism(new_want_determinism);
 }
 
-void Kernel::DoState(PointerWrap& p)
+void EmulationKernel::DoState(PointerWrap& p)
 {
   p.Do(m_request_queue);
   p.Do(m_reply_queue);
@@ -829,6 +860,9 @@ void Kernel::DoState(PointerWrap& p)
   if (m_title_id == Titles::MIOS)
     return;
 
+  if (m_socket_manager)
+    m_socket_manager->DoState(p);
+
   for (const auto& entry : m_device_map)
     entry.second->DoState(p);
 
@@ -897,7 +931,10 @@ static void FinishPPCBootstrap(Core::System& system, u64 userdata, s64 cycles_la
   else
     ReleasePPC();
 
-  SConfig::OnNewTitleLoad();
+  ASSERT(Core::IsCPUThread());
+  Core::CPUThreadGuard guard(system);
+  SConfig::OnNewTitleLoad(guard);
+
   INFO_LOG_FMT(IOS, "Bootstrapping done.");
 }
 
@@ -907,7 +944,7 @@ void Init()
   auto& core_timing = system.GetCoreTiming();
 
   s_event_enqueue =
-      core_timing.RegisterEvent("IPCEvent", [](Core::System& system, u64 userdata, s64) {
+      core_timing.RegisterEvent("IPCEvent", [](Core::System& system_, u64 userdata, s64) {
         if (s_ios)
           s_ios->HandleIPCEvent(userdata);
       });
@@ -917,15 +954,15 @@ void Init()
   s_event_finish_ppc_bootstrap =
       core_timing.RegisterEvent("IOSFinishPPCBootstrap", FinishPPCBootstrap);
 
-  s_event_finish_ios_boot =
-      core_timing.RegisterEvent("IOSFinishIOSBoot", [](Core::System& system, u64 ios_title_id,
-                                                       s64) { FinishIOSBoot(ios_title_id); });
+  s_event_finish_ios_boot = core_timing.RegisterEvent(
+      "IOSFinishIOSBoot",
+      [](Core::System& system_, u64 ios_title_id, s64) { FinishIOSBoot(system_, ios_title_id); });
 
   DIDevice::s_finish_executing_di_command =
       core_timing.RegisterEvent("FinishDICommand", DIDevice::FinishDICommandCallback);
 
   // Start with IOS80 to simulate part of the Wii boot process.
-  s_ios = std::make_unique<EmulationKernel>(Titles::SYSTEM_MENU_IOS);
+  s_ios = std::make_unique<EmulationKernel>(system, Titles::SYSTEM_MENU_IOS);
   // On a Wii, boot2 launches the system menu IOS, which then launches the system menu
   // (which bootstraps the PPC). Bootstrapping the PPC results in memory values being set up.
   // This means that the constants in the 0x3100 region are always set up by the time
