@@ -21,11 +21,13 @@
 #include "Core/HW/Memmap.h"
 #include "Core/IOS/IOS.h"
 #include "Core/IOS/VersionInfo.h"
+#include "Core/System.h"
 
 namespace IOS::HLE
 {
-SDIOSlot0Device::SDIOSlot0Device(Kernel& ios, const std::string& device_name)
-    : Device(ios, device_name), m_sdhc_supported(HasFeature(ios.GetVersion(), Feature::SDv2))
+SDIOSlot0Device::SDIOSlot0Device(EmulationKernel& ios, const std::string& device_name)
+    : EmulationDevice(ios, device_name),
+      m_sdhc_supported(HasFeature(ios.GetVersion(), Feature::SDv2))
 {
   if (!Config::Get(Config::MAIN_ALLOW_SD_WRITES))
     INFO_LOG_FMT(IOS_SD, "Writes to SD card disabled by user");
@@ -82,7 +84,7 @@ void SDIOSlot0Device::EventNotify()
     else
       INFO_LOG_FMT(IOS_SD, "Notifying PPC of SD card removal");
 
-    m_ios.EnqueueIPCReply(m_event->request, m_event->type);
+    GetEmulationKernel().EnqueueIPCReply(m_event->request, m_event->type);
     m_event.reset();
   }
 }
@@ -128,7 +130,9 @@ std::optional<IPCReply> SDIOSlot0Device::Close(u32 fd)
 
 std::optional<IPCReply> SDIOSlot0Device::IOCtl(const IOCtlRequest& request)
 {
-  Memory::Memset(request.buffer_out, 0, request.buffer_out_size);
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+  memory.Memset(request.buffer_out, 0, request.buffer_out_size);
 
   switch (request.request)
   {
@@ -187,15 +191,18 @@ s32 SDIOSlot0Device::ExecuteCommand(const Request& request, u32 buffer_in, u32 b
     u32 pad0;
   } req;
 
-  req.command = Memory::Read_U32(buffer_in + 0);
-  req.type = Memory::Read_U32(buffer_in + 4);
-  req.resp = Memory::Read_U32(buffer_in + 8);
-  req.arg = Memory::Read_U32(buffer_in + 12);
-  req.blocks = Memory::Read_U32(buffer_in + 16);
-  req.bsize = Memory::Read_U32(buffer_in + 20);
-  req.addr = Memory::Read_U32(buffer_in + 24);
-  req.isDMA = Memory::Read_U32(buffer_in + 28);
-  req.pad0 = Memory::Read_U32(buffer_in + 32);
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+
+  req.command = memory.Read_U32(buffer_in + 0);
+  req.type = memory.Read_U32(buffer_in + 4);
+  req.resp = memory.Read_U32(buffer_in + 8);
+  req.arg = memory.Read_U32(buffer_in + 12);
+  req.blocks = memory.Read_U32(buffer_in + 16);
+  req.bsize = memory.Read_U32(buffer_in + 20);
+  req.addr = memory.Read_U32(buffer_in + 24);
+  req.isDMA = memory.Read_U32(buffer_in + 28);
+  req.pad0 = memory.Read_U32(buffer_in + 32);
 
   // Note: req.addr is the virtual address of _rwBuffer
 
@@ -206,19 +213,19 @@ s32 SDIOSlot0Device::ExecuteCommand(const Request& request, u32 buffer_in, u32 b
   case GO_IDLE_STATE:
     INFO_LOG_FMT(IOS_SD, "GO_IDLE_STATE");
     // Response is R1 (idle state)
-    Memory::Write_U32(0x00, buffer_out);
+    memory.Write_U32(0x00, buffer_out);
     break;
 
   case SEND_RELATIVE_ADDR:
     // Technically RCA should be generated when asked and at power on...w/e :p
-    Memory::Write_U32(0x9f62, buffer_out);
+    memory.Write_U32(0x9f62, buffer_out);
     break;
 
   case SELECT_CARD:
     // This covers both select and deselect
     // Differentiate by checking if rca is set in req.arg
     // If it is, it's a select and return 0x700
-    Memory::Write_U32((req.arg >> 16) ? 0x700 : 0x900, buffer_out);
+    memory.Write_U32((req.arg >> 16) ? 0x700 : 0x900, buffer_out);
     break;
 
   case SEND_IF_COND:
@@ -227,45 +234,45 @@ s32 SDIOSlot0Device::ExecuteCommand(const Request& request, u32 buffer_in, u32 b
     // voltage and the check pattern that were set in the command argument.
     // This command is used to differentiate between protocol v1 and v2.
     InitSDHC();
-    Memory::Write_U32(req.arg, buffer_out);
+    memory.Write_U32(req.arg, buffer_out);
     break;
 
   case SEND_CSD:
   {
     const std::array<u32, 4> csd = m_protocol == SDProtocol::V1 ? GetCSDv1() : GetCSDv2();
-    Memory::CopyToEmuSwapped(buffer_out, csd.data(), csd.size() * sizeof(u32));
+    memory.CopyToEmuSwapped(buffer_out, csd.data(), csd.size() * sizeof(u32));
   }
   break;
 
   case ALL_SEND_CID:
   case SEND_CID:
     INFO_LOG_FMT(IOS_SD, "(ALL_)SEND_CID");
-    Memory::Write_U32(0x80114d1c, buffer_out);
-    Memory::Write_U32(0x80080000, buffer_out + 4);
-    Memory::Write_U32(0x8007b520, buffer_out + 8);
-    Memory::Write_U32(0x80080000, buffer_out + 12);
+    memory.Write_U32(0x80114d1c, buffer_out);
+    memory.Write_U32(0x80080000, buffer_out + 4);
+    memory.Write_U32(0x8007b520, buffer_out + 8);
+    memory.Write_U32(0x80080000, buffer_out + 12);
     break;
 
   case SET_BLOCKLEN:
     m_block_length = req.arg;
-    Memory::Write_U32(0x900, buffer_out);
+    memory.Write_U32(0x900, buffer_out);
     break;
 
   case APP_CMD_NEXT:
     // Next cmd is going to be ACMD_*
-    Memory::Write_U32(0x920, buffer_out);
+    memory.Write_U32(0x920, buffer_out);
     break;
 
   case ACMD_SETBUSWIDTH:
     // 0 = 1bit, 2 = 4bit
     m_bus_width = (req.arg & 3);
-    Memory::Write_U32(0x920, buffer_out);
+    memory.Write_U32(0x920, buffer_out);
     break;
 
   case ACMD_SENDOPCOND:
     // Sends host capacity support information (HCS) and asks the accessed card to send
     // its operating condition register (OCR) content
-    Memory::Write_U32(GetOCRegister(), buffer_out);
+    memory.Write_U32(GetOCRegister(), buffer_out);
     break;
 
   case READ_MULTIPLE_BLOCK:
@@ -283,7 +290,7 @@ s32 SDIOSlot0Device::ExecuteCommand(const Request& request, u32 buffer_in, u32 b
       if (!m_card.Seek(address, File::SeekOrigin::Begin))
         ERROR_LOG_FMT(IOS_SD, "Seek failed");
 
-      if (m_card.ReadBytes(Memory::GetPointer(req.addr), size))
+      if (m_card.ReadBytes(memory.GetPointer(req.addr), size))
       {
         DEBUG_LOG_FMT(IOS_SD, "Outbuffer size {} got {}", rw_buffer_size, size);
       }
@@ -295,7 +302,7 @@ s32 SDIOSlot0Device::ExecuteCommand(const Request& request, u32 buffer_in, u32 b
       }
     }
   }
-    Memory::Write_U32(0x900, buffer_out);
+    memory.Write_U32(0x900, buffer_out);
     break;
 
   case WRITE_MULTIPLE_BLOCK:
@@ -313,7 +320,7 @@ s32 SDIOSlot0Device::ExecuteCommand(const Request& request, u32 buffer_in, u32 b
       if (!m_card.Seek(address, File::SeekOrigin::Begin))
         ERROR_LOG_FMT(IOS_SD, "Seek failed");
 
-      if (!m_card.WriteBytes(Memory::GetPointer(req.addr), size))
+      if (!m_card.WriteBytes(memory.GetPointer(req.addr), size))
       {
         ERROR_LOG_FMT(IOS_SD, "Write Failed - error: {}, eof: {}", std::ferror(m_card.GetHandle()),
                       std::feof(m_card.GetHandle()));
@@ -321,7 +328,7 @@ s32 SDIOSlot0Device::ExecuteCommand(const Request& request, u32 buffer_in, u32 b
       }
     }
   }
-    Memory::Write_U32(0x900, buffer_out);
+    memory.Write_U32(0x900, buffer_out);
     break;
 
   case EVENT_REGISTER:  // async
@@ -339,7 +346,7 @@ s32 SDIOSlot0Device::ExecuteCommand(const Request& request, u32 buffer_in, u32 b
     // release returns 0
     // unknown sd int
     // technically we do it out of order, oh well
-    m_ios.EnqueueIPCReply(m_event->request, EVENT_INVALID);
+    GetEmulationKernel().EnqueueIPCReply(m_event->request, EVENT_INVALID);
     m_event.reset();
     break;
   }
@@ -354,8 +361,11 @@ s32 SDIOSlot0Device::ExecuteCommand(const Request& request, u32 buffer_in, u32 b
 
 IPCReply SDIOSlot0Device::WriteHCRegister(const IOCtlRequest& request)
 {
-  const u32 reg = Memory::Read_U32(request.buffer_in);
-  const u32 val = Memory::Read_U32(request.buffer_in + 16);
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+
+  const u32 reg = memory.Read_U32(request.buffer_in);
+  const u32 val = memory.Read_U32(request.buffer_in + 16);
 
   INFO_LOG_FMT(IOS_SD, "IOCTL_WRITEHCR {:#010x} - {:#010x}", reg, val);
 
@@ -386,7 +396,10 @@ IPCReply SDIOSlot0Device::WriteHCRegister(const IOCtlRequest& request)
 
 IPCReply SDIOSlot0Device::ReadHCRegister(const IOCtlRequest& request)
 {
-  const u32 reg = Memory::Read_U32(request.buffer_in);
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+
+  const u32 reg = memory.Read_U32(request.buffer_in);
 
   if (reg >= m_registers.size())
   {
@@ -398,7 +411,7 @@ IPCReply SDIOSlot0Device::ReadHCRegister(const IOCtlRequest& request)
   INFO_LOG_FMT(IOS_SD, "IOCTL_READHCR {:#010x} - {:#010x}", reg, val);
 
   // Just reading the register
-  Memory::Write_U32(val, request.buffer_out);
+  memory.Write_U32(val, request.buffer_out);
   return IPCReply(IPC_SUCCESS);
 }
 
@@ -406,8 +419,11 @@ IPCReply SDIOSlot0Device::ResetCard(const IOCtlRequest& request)
 {
   INFO_LOG_FMT(IOS_SD, "IOCTL_RESETCARD");
 
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+
   // Returns 16bit RCA and 16bit 0s (meaning success)
-  Memory::Write_U32(m_status, request.buffer_out);
+  memory.Write_U32(m_status, request.buffer_out);
 
   return IPCReply(IPC_SUCCESS);
 }
@@ -416,9 +432,12 @@ IPCReply SDIOSlot0Device::SetClk(const IOCtlRequest& request)
 {
   INFO_LOG_FMT(IOS_SD, "IOCTL_SETCLK");
 
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+
   // libogc only sets it to 1 and makes sure the return isn't negative...
   // one half of the sdclk divisor: a power of two or zero.
-  const u32 clock = Memory::Read_U32(request.buffer_in);
+  const u32 clock = memory.Read_U32(request.buffer_in);
   if (clock != 1)
     INFO_LOG_FMT(IOS_SD, "Setting to {}, interesting", clock);
 
@@ -427,7 +446,10 @@ IPCReply SDIOSlot0Device::SetClk(const IOCtlRequest& request)
 
 std::optional<IPCReply> SDIOSlot0Device::SendCommand(const IOCtlRequest& request)
 {
-  INFO_LOG_FMT(IOS_SD, "IOCTL_SENDCMD {:x} IPC:{:08x}", Memory::Read_U32(request.buffer_in),
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+
+  INFO_LOG_FMT(IOS_SD, "IOCTL_SENDCMD {:x} IPC:{:08x}", memory.Read_U32(request.buffer_in),
                request.address);
 
   const s32 return_value = ExecuteCommand(request, request.buffer_in, request.buffer_in_size, 0, 0,
@@ -476,23 +498,31 @@ IPCReply SDIOSlot0Device::GetStatus(const IOCtlRequest& request)
                (status & CARD_INSERTED) ? "inserted" : "not present",
                (status & CARD_INITIALIZED) ? " and initialized" : "");
 
-  Memory::Write_U32(status, request.buffer_out);
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+  memory.Write_U32(status, request.buffer_out);
   return IPCReply(IPC_SUCCESS);
 }
 
 IPCReply SDIOSlot0Device::GetOCRegister(const IOCtlRequest& request)
 {
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+
   const u32 ocr = GetOCRegister();
   INFO_LOG_FMT(IOS_SD, "IOCTL_GETOCR. Replying with ocr {:x}", ocr);
-  Memory::Write_U32(ocr, request.buffer_out);
+  memory.Write_U32(ocr, request.buffer_out);
 
   return IPCReply(IPC_SUCCESS);
 }
 
 IPCReply SDIOSlot0Device::SendCommand(const IOCtlVRequest& request)
 {
-  DEBUG_LOG_FMT(IOS_SD, "IOCTLV_SENDCMD {:#010x}", Memory::Read_U32(request.in_vectors[0].address));
-  Memory::Memset(request.io_vectors[0].address, 0, request.io_vectors[0].size);
+  auto& system = GetSystem();
+  auto& memory = system.GetMemory();
+
+  DEBUG_LOG_FMT(IOS_SD, "IOCTLV_SENDCMD {:#010x}", memory.Read_U32(request.in_vectors[0].address));
+  memory.Memset(request.io_vectors[0].address, 0, request.io_vectors[0].size);
 
   const s32 return_value =
       ExecuteCommand(request, request.in_vectors[0].address, request.in_vectors[0].size,
