@@ -3,6 +3,8 @@
 
 #include "Core/PowerPC/JitArm64/Jit.h"
 
+#include <bit>
+
 #include "Common/Arm64Emitter.h"
 #include "Common/Assert.h"
 #include "Common/BitUtils.h"
@@ -11,6 +13,7 @@
 
 #include "Core/Core.h"
 #include "Core/CoreTiming.h"
+#include "Core/PowerPC/Interpreter/Interpreter.h"
 #include "Core/PowerPC/JitArm64/JitArm64_RegCache.h"
 #include "Core/PowerPC/JitCommon/DivUtils.h"
 #include "Core/PowerPC/PPCTables.h"
@@ -526,7 +529,7 @@ void JitArm64::cntlzwx(UGeckoInstruction inst)
 
   if (gpr.IsImm(s))
   {
-    gpr.SetImmediate(a, Common::CountLeadingZeros(gpr.GetImm(s)));
+    gpr.SetImmediate(a, static_cast<u32>(std::countl_zero(gpr.GetImm(s))));
     if (inst.Rc)
       ComputeRC0(gpr.GetImm(a));
   }
@@ -582,13 +585,13 @@ void JitArm64::cmp(UGeckoInstruction inst)
   }
   else if (gpr.IsImm(a) && !gpr.GetImm(a))
   {
-    NEG(EncodeRegTo32(CR), gpr.R(b));
-    SXTW(CR, EncodeRegTo32(CR));
+    SXTW(CR, gpr.R(b));
+    NEG(CR, CR);
   }
   else if (gpr.IsImm(a) && gpr.GetImm(a) == 0xFFFFFFFF)
   {
-    MVN(EncodeRegTo32(CR), gpr.R(b));
-    SXTW(CR, EncodeRegTo32(CR));
+    SXTW(CR, gpr.R(b));
+    MVN(CR, CR);
   }
   else if (gpr.IsImm(b) && !gpr.GetImm(b))
   {
@@ -700,7 +703,7 @@ void JitArm64::rlwinmx(UGeckoInstruction inst)
   const u32 mask = MakeRotationMask(inst.MB, inst.ME);
   if (gpr.IsImm(inst.RS))
   {
-    gpr.SetImmediate(a, Common::RotateLeft(gpr.GetImm(s), inst.SH) & mask);
+    gpr.SetImmediate(a, std::rotl(gpr.GetImm(s), inst.SH) & mask);
     if (inst.Rc)
       ComputeRC0(gpr.GetImm(a));
     return;
@@ -749,7 +752,7 @@ void JitArm64::rlwnmx(UGeckoInstruction inst)
 
   if (gpr.IsImm(b) && gpr.IsImm(s))
   {
-    gpr.SetImmediate(a, Common::RotateLeft(gpr.GetImm(s), gpr.GetImm(b) & 0x1F) & mask);
+    gpr.SetImmediate(a, std::rotl(gpr.GetImm(s), gpr.GetImm(b) & 0x1F) & mask);
     if (inst.Rc)
       ComputeRC0(gpr.GetImm(a));
   }
@@ -907,7 +910,7 @@ bool JitArm64::MultiplyImmediate(u32 imm, int a, int d, bool rc)
   else if (MathUtil::IsPow2(imm))
   {
     // Multiplication by a power of two (2^n).
-    const int shift = IntLog2(imm);
+    const int shift = MathUtil::IntLog2(imm);
 
     gpr.BindToRegister(d, d == a);
     LSL(gpr.R(d), gpr.R(a), shift);
@@ -917,7 +920,7 @@ bool JitArm64::MultiplyImmediate(u32 imm, int a, int d, bool rc)
   else if (MathUtil::IsPow2(imm - 1))
   {
     // Multiplication by a power of two plus one (2^n + 1).
-    const int shift = IntLog2(imm - 1);
+    const int shift = MathUtil::IntLog2(imm - 1);
 
     gpr.BindToRegister(d, d == a);
     ADD(gpr.R(d), gpr.R(a), gpr.R(a), ArithOption(gpr.R(a), ShiftType::LSL, shift));
@@ -927,7 +930,7 @@ bool JitArm64::MultiplyImmediate(u32 imm, int a, int d, bool rc)
   else if (MathUtil::IsPow2(~imm + 1))
   {
     // Multiplication by a negative power of two (-(2^n)).
-    const int shift = IntLog2(~imm + 1);
+    const int shift = MathUtil::IntLog2(~imm + 1);
 
     gpr.BindToRegister(d, d == a);
     NEG(gpr.R(d), gpr.R(a), ArithOption(gpr.R(a), ShiftType::LSL, shift));
@@ -937,7 +940,7 @@ bool JitArm64::MultiplyImmediate(u32 imm, int a, int d, bool rc)
   else if (MathUtil::IsPow2(~imm + 2))
   {
     // Multiplication by a negative power of two plus one (-(2^n) + 1).
-    const int shift = IntLog2(~imm + 2);
+    const int shift = MathUtil::IntLog2(~imm + 2);
 
     gpr.BindToRegister(d, d == a);
     SUB(gpr.R(d), gpr.R(a), gpr.R(a), ArithOption(gpr.R(a), ShiftType::LSL, shift));
@@ -1601,9 +1604,9 @@ void JitArm64::divwx(UGeckoInstruction inst)
       CSEL(WA, RA, WA, CCFlags::CC_PL);
 
       if (divisor < 0)
-        NEG(RD, WA, ArithOption(WA, ShiftType::ASR, IntLog2(abs_val)));
+        NEG(RD, WA, ArithOption(WA, ShiftType::ASR, MathUtil::IntLog2(abs_val)));
       else
-        ASR(RD, WA, IntLog2(abs_val));
+        ASR(RD, WA, MathUtil::IntLog2(abs_val));
 
       if (allocate_reg)
         gpr.Unlock(WA);
@@ -1924,7 +1927,7 @@ void JitArm64::rlwimix(UGeckoInstruction inst)
 
   if (gpr.IsImm(a) && gpr.IsImm(s))
   {
-    u32 res = (gpr.GetImm(a) & ~mask) | (Common::RotateLeft(gpr.GetImm(s), inst.SH) & mask);
+    u32 res = (gpr.GetImm(a) & ~mask) | (std::rotl(gpr.GetImm(s), inst.SH) & mask);
     gpr.SetImmediate(a, res);
     if (inst.Rc)
       ComputeRC0(res);
