@@ -12,6 +12,11 @@
 
 #include "Core/Debugger/PPCDebugInterface.h"
 
+namespace Core
+{
+class CPUThreadGuard;
+}
+
 // This has functionality overlapping Debugger_Symbolmap. Should merge that stuff in here later.
 class PPCSymbolDB : public Common::SymbolDB
 {
@@ -19,8 +24,9 @@ public:
   PPCSymbolDB();
   ~PPCSymbolDB() override;
 
-  Common::Symbol* AddFunction(u32 start_addr) override;
-  void AddKnownSymbol(u32 startAddr, u32 size, const std::string& name,
+  Common::Symbol* AddFunction(const Core::CPUThreadGuard& guard, u32 start_addr) override;
+  void AddKnownSymbol(const Core::CPUThreadGuard& guard, u32 startAddr, u32 size,
+                      const std::string& name,
                       Common::Symbol::Type type = Common::Symbol::Type::Function);
 
   Common::Symbol* GetSymbolFromAddr(u32 addr) override;
@@ -29,9 +35,9 @@ public:
 
   void FillInCallers();
 
-  bool LoadMap(const std::string& filename, bool bad = false);
+  bool LoadMap(const Core::CPUThreadGuard& guard, const std::string& filename, bool bad = false);
   bool SaveSymbolMap(const std::string& filename) const;
-  bool SaveCodeMap(const std::string& filename) const;
+  bool SaveCodeMap(const Core::CPUThreadGuard& guard, const std::string& filename) const;
 
   void PrintCalls(u32 funcAddr) const;
   void PrintCallers(u32 funcAddr) const;
