@@ -2,14 +2,11 @@
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "DolphinNoGUI/Platform.h"
+#include "Core/HW/ProcessorInterface.h"
 #include "Core/IOS/IOS.h"
 #include "Core/IOS/STM/STM.h"
 #include "Core/State.h"
-
-namespace ProcessorInterface
-{
-void PowerButton_Tap();
-}
+#include "Core/System.h"
 
 Platform::~Platform() = default;
 
@@ -31,7 +28,8 @@ void Platform::UpdateRunningFlag()
     if (!m_tried_graceful_shutdown.IsSet() && stm &&
         std::static_pointer_cast<IOS::HLE::STMEventHookDevice>(stm)->HasHookInstalled())
     {
-      ProcessorInterface::PowerButton_Tap();
+      auto& system = Core::System::GetInstance();
+      system.GetProcessorInterface().PowerButton_Tap();
       m_tried_graceful_shutdown.Set();
     }
     else
