@@ -3,7 +3,6 @@
 
 #include "DolphinQt/CheatSearchWidget.h"
 
-#include <cassert>
 #include <functional>
 #include <optional>
 #include <string>
@@ -36,7 +35,9 @@
 #include "Core/CheatGeneration.h"
 #include "Core/CheatSearch.h"
 #include "Core/ConfigManager.h"
+#include "Core/Core.h"
 #include "Core/PowerPC/PowerPC.h"
+#include "Core/System.h"
 
 #include "DolphinQt/Config/CheatCodeEditor.h"
 #include "DolphinQt/Config/CheatWarningWidget.h"
@@ -286,7 +287,11 @@ void CheatSearchWidget::OnNextScanClicked()
       return;
     }
   }
-  Cheats::SearchErrorCode error_code = m_session->RunSearch();
+
+  const Cheats::SearchErrorCode error_code = [this] {
+    Core::CPUThreadGuard guard(Core::System::GetInstance());
+    return m_session->RunSearch(guard);
+  }();
 
   if (error_code == Cheats::SearchErrorCode::Success)
   {
@@ -391,7 +396,13 @@ bool CheatSearchWidget::RefreshValues()
   }
 
   tmp->SetFilterType(Cheats::FilterType::DoNotFilter);
-  if (tmp->RunSearch() != Cheats::SearchErrorCode::Success)
+
+  const Cheats::SearchErrorCode error_code = [&tmp] {
+    Core::CPUThreadGuard guard(Core::System::GetInstance());
+    return tmp->RunSearch(guard);
+  }();
+
+  if (error_code != Cheats::SearchErrorCode::Success)
   {
     m_info_label_1->setText(tr("Refresh failed. Please run the game for a bit and try again."));
     return false;
@@ -454,6 +465,10 @@ void CheatSearchWidget::OnAddressTableContextMenu()
   QMenu* menu = new QMenu(this);
 
   menu->addAction(tr("Show in memory"), [this, address] { emit ShowMemory(address); });
+  menu->addAction(tr("Add to watch"), this, [this, address] {
+    const QString name = QStringLiteral("mem_%1").arg(address, 8, 16, QLatin1Char('0'));
+    emit RequestWatch(name, address);
+  });
   menu->addAction(tr("Generate Action Replay Code"), this, &CheatSearchWidget::GenerateARCode);
 
   menu->exec(QCursor::pos());
