@@ -58,6 +58,8 @@ private:
 
   void ClearLayoutRecursively(QLayout* layout);
 
+  void OpenGraphicsModDir();
+
   void CalculateGameRunning(Core::State state);
   bool m_loaded_game_is_running = false;
   bool m_needs_save = false;
@@ -67,6 +69,7 @@ private:
   QLabel* m_selected_mod_name;
   QVBoxLayout* m_mod_meta_layout;
 
+  QPushButton* m_open_directory_button;
   QPushButton* m_refresh;
   GraphicsModWarningWidget* m_warning;
 
