@@ -7,10 +7,12 @@
 #include <string>
 #include <vector>
 
+#include <QGuiApplication>
 #include <QHBoxLayout>
 #include <QLabel>
 #include <QMenu>
 #include <QPushButton>
+#include <QStyleHints>
 #include <QTableWidget>
 #include <QVBoxLayout>
 
@@ -23,12 +25,17 @@
 #include "Core/PowerPC/PPCSymbolDB.h"
 #include "Core/PowerPC/PowerPC.h"
 #include "Core/PowerPC/Profiler.h"
+#include "Core/System.h"
 
 #include "DolphinQt/Debugger/CodeWidget.h"
 #include "DolphinQt/Host.h"
 #include "DolphinQt/QtUtils/ModalMessageBox.h"
 #include "DolphinQt/Settings.h"
 
+static const QString RECORD_BUTTON_STYLESHEET =
+    QStringLiteral("QPushButton:checked { background-color: rgb(150, 0, 0); border-style: solid; "
+                   "border-width: 3px; border-color: rgb(150,0,0); color: rgb(255, 255, 255);}");
+
 CodeDiffDialog::CodeDiffDialog(CodeWidget* parent) : QDialog(parent), m_code_widget(parent)
 {
   setWindowTitle(tr("Code Diff Tool"));
@@ -53,9 +60,7 @@ void CodeDiffDialog::CreateWidgets()
   m_include_btn = new QPushButton(tr("Code has been executed"));
   m_record_btn = new QPushButton(tr("Start Recording"));
   m_record_btn->setCheckable(true);
-  m_record_btn->setStyleSheet(
-      QStringLiteral("QPushButton:checked { background-color: rgb(150, 0, 0); border-style: solid; "
-                     "border-width: 3px; border-color: rgb(150,0,0); color: rgb(255, 255, 255);}"));
+  m_record_btn->setStyleSheet(RECORD_BUTTON_STYLESHEET);
 
   m_exclude_btn->setEnabled(false);
   m_include_btn->setEnabled(false);
@@ -104,6 +109,13 @@ void CodeDiffDialog::CreateWidgets()
 
 void CodeDiffDialog::ConnectWidgets()
 {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 5, 0)
+  connect(QGuiApplication::styleHints(), &QStyleHints::colorSchemeChanged, this,
+          [this](Qt::ColorScheme colorScheme) {
+            m_record_btn->setStyleSheet(RECORD_BUTTON_STYLESHEET);
+          });
+#endif
+
   connect(m_record_btn, &QPushButton::toggled, this, &CodeDiffDialog::OnRecord);
   connect(m_include_btn, &QPushButton::pressed, [this]() { Update(true); });
   connect(m_exclude_btn, &QPushButton::pressed, [this]() { Update(false); });
@@ -139,7 +151,8 @@ void CodeDiffDialog::ClearData()
   // Swap is used instead of clear for efficiency in the case of huge m_include/m_exclude
   std::vector<Diff>().swap(m_include);
   std::vector<Diff>().swap(m_exclude);
-  JitInterface::SetProfilingState(JitInterface::ProfilingState::Disabled);
+  Core::System::GetInstance().GetJitInterface().SetProfilingState(
+      JitInterface::ProfilingState::Disabled);
 }
 
 void CodeDiffDialog::ClearBlockCache()
@@ -149,7 +162,7 @@ void CodeDiffDialog::ClearBlockCache()
   if (old_state == Core::State::Running)
     Core::SetState(Core::State::Paused);
 
-  JitInterface::ClearCache();
+  Core::System::GetInstance().GetJitInterface().ClearCache();
 
   if (old_state == Core::State::Running)
     Core::SetState(Core::State::Running);
@@ -204,7 +217,7 @@ void CodeDiffDialog::OnRecord(bool enabled)
   }
 
   m_record_btn->update();
-  JitInterface::SetProfilingState(state);
+  Core::System::GetInstance().GetJitInterface().SetProfilingState(state);
 }
 
 void CodeDiffDialog::OnInclude()
@@ -270,7 +283,7 @@ std::vector<Diff> CodeDiffDialog::CalculateSymbolsFromProfile()
 {
   Profiler::ProfileStats prof_stats;
   auto& blockstats = prof_stats.block_stats;
-  JitInterface::GetProfileResults(&prof_stats);
+  Core::System::GetInstance().GetJitInterface().GetProfileResults(&prof_stats);
   std::vector<Diff> current;
   current.reserve(20000);
 
@@ -390,7 +403,7 @@ void CodeDiffDialog::Update(bool include)
   m_exclude_size_label->setText(tr("Excluded: %1").arg(m_exclude.size()));
   m_include_size_label->setText(tr("Included: %1").arg(m_include.size()));
 
-  JitInterface::ClearCache();
+  Core::System::GetInstance().GetJitInterface().ClearCache();
   if (old_state == Core::State::Running)
     Core::SetState(Core::State::Running);
 }
@@ -483,7 +496,12 @@ void CodeDiffDialog::OnSetBLR()
   Common::Symbol* symbol = g_symbolDB.GetSymbolFromAddr(item->data(Qt::UserRole).toUInt());
   if (!symbol)
     return;
-  PowerPC::debug_interface.SetPatch(symbol->address, 0x4E800020);
+
+  {
+    auto& system = Core::System::GetInstance();
+    Core::CPUThreadGuard guard(system);
+    system.GetPowerPC().GetDebugInterface().SetPatch(guard, symbol->address, 0x4E800020);
+  }
 
   int row = item->row();
   m_matching_results_table->item(row, 0)->setForeground(QBrush(Qt::red));
