@@ -15,6 +15,7 @@
 #include "Core/Core.h"
 #include "Core/PowerPC/MMU.h"
 #include "Core/PowerPC/PowerPC.h"
+#include "Core/System.h"
 #include "DolphinQt/Host.h"
 #include "DolphinQt/Settings.h"
 
@@ -256,7 +257,9 @@ void ThreadWidget::Update()
   {
     m_thread_table->setRowCount(0);
     UpdateThreadContext({});
-    UpdateThreadCallstack({});
+
+    Core::CPUThreadGuard guard(Core::System::GetInstance());
+    UpdateThreadCallstack(guard, {});
   }
   if (emu_state != Core::State::Paused)
     return;
@@ -264,8 +267,9 @@ void ThreadWidget::Update()
   const auto format_hex = [](u32 value) {
     return QStringLiteral("%1").arg(value, 8, 16, QLatin1Char('0'));
   };
-  const auto format_hex_from = [&format_hex](u32 addr) {
-    addr = PowerPC::HostIsRAMAddress(addr) ? PowerPC::HostRead_U32(addr) : 0;
+  const auto format_hex_from = [&format_hex](const Core::CPUThreadGuard& guard, u32 addr) {
+    addr =
+        PowerPC::MMU::HostIsRAMAddress(guard, addr) ? PowerPC::MMU::HostRead_U32(guard, addr) : 0;
     return format_hex(addr);
   };
   const auto get_state = [](u16 thread_state) {
@@ -298,35 +302,41 @@ void ThreadWidget::Update()
         .arg(start, 8, 16, QLatin1Char('0'));
   };
 
-  // YAGCD - Section 4.2.1.4 Dolphin OS Globals
-  m_current_context->setText(format_hex_from(0x800000D4));
-  m_current_thread->setText(format_hex_from(0x800000E4));
-  m_default_thread->setText(format_hex_from(0x800000D8));
-
-  m_queue_head->setText(format_hex_from(0x800000DC));
-  m_queue_tail->setText(format_hex_from(0x800000E0));
-
-  // Thread group
-  m_threads = PowerPC::debug_interface.GetThreads();
-  int i = 0;
-  m_thread_table->setRowCount(i);
-  for (const auto& thread : m_threads)
   {
-    m_thread_table->insertRow(i);
-    m_thread_table->setItem(i, 0, new QTableWidgetItem(format_hex(thread->GetAddress())));
-    m_thread_table->setItem(i, 1, new QTableWidgetItem(get_state(thread->GetState())));
-    m_thread_table->setItem(i, 2, new QTableWidgetItem(QString::number(thread->IsDetached())));
-    m_thread_table->setItem(i, 3, new QTableWidgetItem(QString::number(thread->IsSuspended())));
-    m_thread_table->setItem(i, 4,
-                            new QTableWidgetItem(get_priority(thread->GetBasePriority(),
-                                                              thread->GetEffectivePriority())));
-    m_thread_table->setItem(
-        i, 5, new QTableWidgetItem(get_stack(thread->GetStackEnd(), thread->GetStackStart())));
-    m_thread_table->setItem(i, 6, new QTableWidgetItem(QString::number(thread->GetErrno())));
-    m_thread_table->setItem(i, 7,
-                            new QTableWidgetItem(QString::fromStdString(thread->GetSpecific())));
-    i += 1;
+    Core::CPUThreadGuard guard(Core::System::GetInstance());
+
+    // YAGCD - Section 4.2.1.4 Dolphin OS Globals
+    m_current_context->setText(format_hex_from(guard, 0x800000D4));
+    m_current_thread->setText(format_hex_from(guard, 0x800000E4));
+    m_default_thread->setText(format_hex_from(guard, 0x800000D8));
+
+    m_queue_head->setText(format_hex_from(guard, 0x800000DC));
+    m_queue_tail->setText(format_hex_from(guard, 0x800000E0));
+
+    // Thread group
+    m_threads = guard.GetSystem().GetPowerPC().GetDebugInterface().GetThreads(guard);
+
+    int i = 0;
+    m_thread_table->setRowCount(i);
+    for (const auto& thread : m_threads)
+    {
+      m_thread_table->insertRow(i);
+      m_thread_table->setItem(i, 0, new QTableWidgetItem(format_hex(thread->GetAddress())));
+      m_thread_table->setItem(i, 1, new QTableWidgetItem(get_state(thread->GetState())));
+      m_thread_table->setItem(i, 2, new QTableWidgetItem(QString::number(thread->IsDetached())));
+      m_thread_table->setItem(i, 3, new QTableWidgetItem(QString::number(thread->IsSuspended())));
+      m_thread_table->setItem(i, 4,
+                              new QTableWidgetItem(get_priority(thread->GetBasePriority(),
+                                                                thread->GetEffectivePriority())));
+      m_thread_table->setItem(
+          i, 5, new QTableWidgetItem(get_stack(thread->GetStackEnd(), thread->GetStackStart())));
+      m_thread_table->setItem(i, 6, new QTableWidgetItem(QString::number(thread->GetErrno())));
+      m_thread_table->setItem(
+          i, 7, new QTableWidgetItem(QString::fromStdString(thread->GetSpecific(guard))));
+      i += 1;
+    }
   }
+
   m_thread_table->resizeColumnsToContents();
   m_thread_table->resizeRowsToContents();
 
@@ -425,7 +435,8 @@ void ThreadWidget::UpdateThreadContext(const Common::Debug::PartialContext& cont
   m_context_table->resizeColumnsToContents();
 }
 
-void ThreadWidget::UpdateThreadCallstack(const Common::Debug::PartialContext& context)
+void ThreadWidget::UpdateThreadCallstack(const Core::CPUThreadGuard& guard,
+                                         const Common::Debug::PartialContext& context)
 {
   m_callstack_table->setRowCount(0);
 
@@ -439,34 +450,36 @@ void ThreadWidget::UpdateThreadCallstack(const Common::Debug::PartialContext& co
   u32 sp = context.gpr->at(1);
   for (int i = 0; i < 16; i++)
   {
-    if (sp == 0 || sp == 0xffffffff || !PowerPC::HostIsRAMAddress(sp))
+    if (sp == 0 || sp == 0xffffffff || !PowerPC::MMU::HostIsRAMAddress(guard, sp))
       break;
     m_callstack_table->insertRow(i);
     m_callstack_table->setItem(i, 0, new QTableWidgetItem(format_hex(sp)));
-    if (PowerPC::HostIsRAMAddress(sp + 4))
+    if (PowerPC::MMU::HostIsRAMAddress(guard, sp + 4))
     {
-      const u32 lr_save = PowerPC::HostRead_U32(sp + 4);
+      const u32 lr_save = PowerPC::MMU::HostRead_U32(guard, sp + 4);
       m_callstack_table->setItem(i, 2, new QTableWidgetItem(format_hex(lr_save)));
-      m_callstack_table->setItem(i, 3,
-                                 new QTableWidgetItem(QString::fromStdString(
-                                     PowerPC::debug_interface.GetDescription(lr_save))));
+      m_callstack_table->setItem(
+          i, 3,
+          new QTableWidgetItem(QString::fromStdString(
+              guard.GetSystem().GetPowerPC().GetDebugInterface().GetDescription(lr_save))));
     }
     else
     {
       m_callstack_table->setItem(i, 2, new QTableWidgetItem(QStringLiteral("--------")));
     }
-    sp = PowerPC::HostRead_U32(sp);
+    sp = PowerPC::MMU::HostRead_U32(guard, sp);
     m_callstack_table->setItem(i, 1, new QTableWidgetItem(format_hex(sp)));
   }
 }
 
 void ThreadWidget::OnSelectionChanged(int row)
 {
+  Core::CPUThreadGuard guard(Core::System::GetInstance());
   Common::Debug::PartialContext context;
 
   if (row >= 0 && size_t(row) < m_threads.size())
-    context = m_threads[row]->GetContext();
+    context = m_threads[row]->GetContext(guard);
 
   UpdateThreadContext(context);
-  UpdateThreadCallstack(context);
+  UpdateThreadCallstack(guard, context);
 }
