@@ -1,11 +1,13 @@
-// Copyright 2008 Dolphin Emulator Project
+// Copyright 2021 Dolphin Emulator Project
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #pragma once
 
 namespace ciface::Android
 {
-void SetMotionSensorsEnabled(bool accelerometer_enabled, bool gyroscope_enabled);
+void Init();
+void Shutdown();
 
 void PopulateDevices();
+
 }  // namespace ciface::Android
