@@ -4,13 +4,16 @@
 #pragma once
 
 #include <string>
-#include <unordered_map>
 #include <vector>
 
 #include "Common/CommonTypes.h"
-#include "Common/IniFile.h"
 #include "InputCommon/DynamicInputTextures/DITData.h"
 
+namespace Common
+{
+class IniFile;
+}
+
 namespace InputCommon::DynamicInputTextures
 {
 class Configuration
@@ -18,11 +21,12 @@ class Configuration
 public:
   explicit Configuration(const std::string& json_file);
   ~Configuration();
-  bool GenerateTextures(const IniFile& file,
+  bool GenerateTextures(const Common::IniFile& file,
                         const std::vector<std::string>& controller_names) const;
 
 private:
-  bool GenerateTexture(const IniFile& file, const std::vector<std::string>& controller_names,
+  bool GenerateTexture(const Common::IniFile& file,
+                       const std::vector<std::string>& controller_names,
                        const Data& texture_data) const;
 
   std::vector<Data> m_dynamic_input_textures;
