@@ -221,30 +221,30 @@ void CommandProcessorManager::RegisterMMIO(Core::System& system, MMIO::Mapping*
                    MMIO::InvalidWrite<u16>());
   }
 
-  mmio->Register(base | STATUS_REGISTER, MMIO::ComplexRead<u16>([](Core::System& system, u32) {
-                   auto& cp = system.GetCommandProcessor();
-                   Fifo::SyncGPUForRegisterAccess();
-                   cp.SetCpStatusRegister();
+  mmio->Register(base | STATUS_REGISTER, MMIO::ComplexRead<u16>([](Core::System& system_, u32) {
+                   auto& cp = system_.GetCommandProcessor();
+                   system_.GetFifo().SyncGPUForRegisterAccess(system_);
+                   cp.SetCpStatusRegister(system_);
                    return cp.m_cp_status_reg.Hex;
                  }),
                  MMIO::InvalidWrite<u16>());
 
   mmio->Register(base | CTRL_REGISTER, MMIO::DirectRead<u16>(&m_cp_ctrl_reg.Hex),
-                 MMIO::ComplexWrite<u16>([](Core::System& system, u32, u16 val) {
-                   auto& cp = system.GetCommandProcessor();
+                 MMIO::ComplexWrite<u16>([](Core::System& system_, u32, u16 val) {
+                   auto& cp = system_.GetCommandProcessor();
                    UCPCtrlReg tmp(val);
                    cp.m_cp_ctrl_reg.Hex = tmp.Hex;
-                   cp.SetCpControlRegister();
-                   Fifo::RunGpu();
+                   cp.SetCpControlRegister(system_);
+                   system_.GetFifo().RunGpu(system_);
                  }));
 
   mmio->Register(base | CLEAR_REGISTER, MMIO::DirectRead<u16>(&m_cp_clear_reg.Hex),
-                 MMIO::ComplexWrite<u16>([](Core::System& system, u32, u16 val) {
-                   auto& cp = system.GetCommandProcessor();
+                 MMIO::ComplexWrite<u16>([](Core::System& system_, u32, u16 val) {
+                   auto& cp = system_.GetCommandProcessor();
                    UCPClearReg tmp(val);
                    cp.m_cp_clear_reg.Hex = tmp.Hex;
                    cp.SetCpClearRegister();
-                   Fifo::RunGpu();
+                   system_.GetFifo().RunGpu(system_);
                  }));
 
   mmio->Register(base | PERF_SELECT, MMIO::InvalidRead<u16>(), MMIO::Nop<u16>());
@@ -254,20 +254,20 @@ void CommandProcessorManager::RegisterMMIO(Core::System& system, MMIO::Mapping*
   MMIO::ReadHandlingMethod<u16>* fifo_rw_distance_lo_r;
   if (is_on_thread)
   {
-    fifo_rw_distance_lo_r = MMIO::ComplexRead<u16>([](Core::System& system, u32) {
-      const auto& fifo = system.GetCommandProcessor().GetFifo();
-      if (fifo.CPWritePointer.load(std::memory_order_relaxed) >=
-          fifo.SafeCPReadPointer.load(std::memory_order_relaxed))
+    fifo_rw_distance_lo_r = MMIO::ComplexRead<u16>([](Core::System& system_, u32) {
+      const auto& fifo_ = system_.GetCommandProcessor().GetFifo();
+      if (fifo_.CPWritePointer.load(std::memory_order_relaxed) >=
+          fifo_.SafeCPReadPointer.load(std::memory_order_relaxed))
       {
-        return static_cast<u16>(fifo.CPWritePointer.load(std::memory_order_relaxed) -
-                                fifo.SafeCPReadPointer.load(std::memory_order_relaxed));
+        return static_cast<u16>(fifo_.CPWritePointer.load(std::memory_order_relaxed) -
+                                fifo_.SafeCPReadPointer.load(std::memory_order_relaxed));
       }
       else
       {
-        return static_cast<u16>(fifo.CPEnd.load(std::memory_order_relaxed) -
-                                fifo.SafeCPReadPointer.load(std::memory_order_relaxed) +
-                                fifo.CPWritePointer.load(std::memory_order_relaxed) -
-                                fifo.CPBase.load(std::memory_order_relaxed) + 32);
+        return static_cast<u16>(fifo_.CPEnd.load(std::memory_order_relaxed) -
+                                fifo_.SafeCPReadPointer.load(std::memory_order_relaxed) +
+                                fifo_.CPWritePointer.load(std::memory_order_relaxed) -
+                                fifo_.CPBase.load(std::memory_order_relaxed) + 32);
       }
     });
   }
@@ -282,40 +282,40 @@ void CommandProcessorManager::RegisterMMIO(Core::System& system, MMIO::Mapping*
   MMIO::ReadHandlingMethod<u16>* fifo_rw_distance_hi_r;
   if (is_on_thread)
   {
-    fifo_rw_distance_hi_r = MMIO::ComplexRead<u16>([](Core::System& system, u32) {
-      const auto& fifo = system.GetCommandProcessor().GetFifo();
-      Fifo::SyncGPUForRegisterAccess();
-      if (fifo.CPWritePointer.load(std::memory_order_relaxed) >=
-          fifo.SafeCPReadPointer.load(std::memory_order_relaxed))
+    fifo_rw_distance_hi_r = MMIO::ComplexRead<u16>([](Core::System& system_, u32) {
+      const auto& fifo_ = system_.GetCommandProcessor().GetFifo();
+      system_.GetFifo().SyncGPUForRegisterAccess(system_);
+      if (fifo_.CPWritePointer.load(std::memory_order_relaxed) >=
+          fifo_.SafeCPReadPointer.load(std::memory_order_relaxed))
       {
-        return (fifo.CPWritePointer.load(std::memory_order_relaxed) -
-                fifo.SafeCPReadPointer.load(std::memory_order_relaxed)) >>
+        return (fifo_.CPWritePointer.load(std::memory_order_relaxed) -
+                fifo_.SafeCPReadPointer.load(std::memory_order_relaxed)) >>
                16;
       }
       else
       {
-        return (fifo.CPEnd.load(std::memory_order_relaxed) -
-                fifo.SafeCPReadPointer.load(std::memory_order_relaxed) +
-                fifo.CPWritePointer.load(std::memory_order_relaxed) -
-                fifo.CPBase.load(std::memory_order_relaxed) + 32) >>
+        return (fifo_.CPEnd.load(std::memory_order_relaxed) -
+                fifo_.SafeCPReadPointer.load(std::memory_order_relaxed) +
+                fifo_.CPWritePointer.load(std::memory_order_relaxed) -
+                fifo_.CPBase.load(std::memory_order_relaxed) + 32) >>
                16;
       }
     });
   }
   else
   {
-    fifo_rw_distance_hi_r = MMIO::ComplexRead<u16>([](Core::System& system, u32) {
-      const auto& fifo = system.GetCommandProcessor().GetFifo();
-      Fifo::SyncGPUForRegisterAccess();
-      return fifo.CPReadWriteDistance.load(std::memory_order_relaxed) >> 16;
+    fifo_rw_distance_hi_r = MMIO::ComplexRead<u16>([](Core::System& system_, u32) {
+      const auto& fifo_ = system_.GetCommandProcessor().GetFifo();
+      system_.GetFifo().SyncGPUForRegisterAccess(system_);
+      return fifo_.CPReadWriteDistance.load(std::memory_order_relaxed) >> 16;
     });
   }
   mmio->Register(base | FIFO_RW_DISTANCE_HI, fifo_rw_distance_hi_r,
-                 MMIO::ComplexWrite<u16>([WMASK_HI_RESTRICT](Core::System& system, u32, u16 val) {
-                   auto& fifo = system.GetCommandProcessor().GetFifo();
-                   Fifo::SyncGPUForRegisterAccess();
-                   WriteHigh(fifo.CPReadWriteDistance, val & WMASK_HI_RESTRICT);
-                   Fifo::RunGpu();
+                 MMIO::ComplexWrite<u16>([WMASK_HI_RESTRICT](Core::System& system_, u32, u16 val) {
+                   auto& fifo_ = system_.GetCommandProcessor().GetFifo();
+                   system_.GetFifo().SyncGPUForRegisterAccess(system_);
+                   WriteHigh(fifo_.CPReadWriteDistance, val & WMASK_HI_RESTRICT);
+                   system_.GetFifo().RunGpu(system_);
                  }));
 
   mmio->Register(
@@ -328,31 +328,33 @@ void CommandProcessorManager::RegisterMMIO(Core::System& system, MMIO::Mapping*
   MMIO::WriteHandlingMethod<u16>* fifo_read_hi_w;
   if (is_on_thread)
   {
-    fifo_read_hi_r = MMIO::ComplexRead<u16>([](Core::System& system, u32) {
-      auto& fifo = system.GetCommandProcessor().GetFifo();
-      Fifo::SyncGPUForRegisterAccess();
-      return fifo.SafeCPReadPointer.load(std::memory_order_relaxed) >> 16;
-    });
-    fifo_read_hi_w = MMIO::ComplexWrite<u16>([WMASK_HI_RESTRICT](Core::System& sys, u32, u16 val) {
-      auto& fifo = sys.GetCommandProcessor().GetFifo();
-      Fifo::SyncGPUForRegisterAccess();
-      WriteHigh(fifo.CPReadPointer, val & WMASK_HI_RESTRICT);
-      fifo.SafeCPReadPointer.store(fifo.CPReadPointer.load(std::memory_order_relaxed),
-                                   std::memory_order_relaxed);
+    fifo_read_hi_r = MMIO::ComplexRead<u16>([](Core::System& system_, u32) {
+      auto& fifo_ = system_.GetCommandProcessor().GetFifo();
+      system_.GetFifo().SyncGPUForRegisterAccess(system_);
+      return fifo_.SafeCPReadPointer.load(std::memory_order_relaxed) >> 16;
     });
+    fifo_read_hi_w =
+        MMIO::ComplexWrite<u16>([WMASK_HI_RESTRICT](Core::System& system_, u32, u16 val) {
+          auto& fifo_ = system_.GetCommandProcessor().GetFifo();
+          system_.GetFifo().SyncGPUForRegisterAccess(system_);
+          WriteHigh(fifo_.CPReadPointer, val & WMASK_HI_RESTRICT);
+          fifo_.SafeCPReadPointer.store(fifo_.CPReadPointer.load(std::memory_order_relaxed),
+                                        std::memory_order_relaxed);
+        });
   }
   else
   {
-    fifo_read_hi_r = MMIO::ComplexRead<u16>([](Core::System& system, u32) {
-      const auto& fifo = system.GetCommandProcessor().GetFifo();
-      Fifo::SyncGPUForRegisterAccess();
-      return fifo.CPReadPointer.load(std::memory_order_relaxed) >> 16;
-    });
-    fifo_read_hi_w = MMIO::ComplexWrite<u16>([WMASK_HI_RESTRICT](Core::System& sys, u32, u16 val) {
-      auto& fifo = sys.GetCommandProcessor().GetFifo();
-      Fifo::SyncGPUForRegisterAccess();
-      WriteHigh(fifo.CPReadPointer, val & WMASK_HI_RESTRICT);
+    fifo_read_hi_r = MMIO::ComplexRead<u16>([](Core::System& system_, u32) {
+      const auto& fifo_ = system_.GetCommandProcessor().GetFifo();
+      system_.GetFifo().SyncGPUForRegisterAccess(system_);
+      return fifo_.CPReadPointer.load(std::memory_order_relaxed) >> 16;
     });
+    fifo_read_hi_w =
+        MMIO::ComplexWrite<u16>([WMASK_HI_RESTRICT](Core::System& system_, u32, u16 val) {
+          auto& fifo_ = system_.GetCommandProcessor().GetFifo();
+          system_.GetFifo().SyncGPUForRegisterAccess(system_);
+          WriteHigh(fifo_.CPReadPointer, val & WMASK_HI_RESTRICT);
+        });
   }
   mmio->Register(base | FIFO_READ_POINTER_HI, fifo_read_hi_r, fifo_read_hi_w);
 }
@@ -363,21 +365,23 @@ void CommandProcessorManager::GatherPipeBursted(Core::System& system)
 
   SetCPStatusFromCPU(system);
 
+  auto& processor_interface = system.GetProcessorInterface();
+
   // if we aren't linked, we don't care about gather pipe data
   if (!m_cp_ctrl_reg.GPLinkEnable)
   {
-    if (IsOnThread(system) && !Fifo::UseDeterministicGPUThread())
+    if (IsOnThread(system) && !system.GetFifo().UseDeterministicGPUThread())
     {
       // In multibuffer mode is not allowed write in the same FIFO attached to the GPU.
       // Fix Pokemon XD in DC mode.
-      if ((ProcessorInterface::Fifo_CPUEnd == fifo.CPEnd.load(std::memory_order_relaxed)) &&
-          (ProcessorInterface::Fifo_CPUBase == fifo.CPBase.load(std::memory_order_relaxed)) &&
+      if ((processor_interface.m_fifo_cpu_end == fifo.CPEnd.load(std::memory_order_relaxed)) &&
+          (processor_interface.m_fifo_cpu_base == fifo.CPBase.load(std::memory_order_relaxed)) &&
           fifo.CPReadWriteDistance.load(std::memory_order_relaxed) > 0)
       {
-        Fifo::FlushGpu();
+        system.GetFifo().FlushGpu(system);
       }
     }
-    Fifo::RunGpu();
+    system.GetFifo().RunGpu(system);
     return;
   }
 
@@ -394,9 +398,10 @@ void CommandProcessorManager::GatherPipeBursted(Core::System& system)
 
   if (m_cp_ctrl_reg.GPReadEnable && m_cp_ctrl_reg.GPLinkEnable)
   {
-    ProcessorInterface::Fifo_CPUWritePointer = fifo.CPWritePointer.load(std::memory_order_relaxed);
-    ProcessorInterface::Fifo_CPUBase = fifo.CPBase.load(std::memory_order_relaxed);
-    ProcessorInterface::Fifo_CPUEnd = fifo.CPEnd.load(std::memory_order_relaxed);
+    processor_interface.m_fifo_cpu_write_pointer =
+        fifo.CPWritePointer.load(std::memory_order_relaxed);
+    processor_interface.m_fifo_cpu_base = fifo.CPBase.load(std::memory_order_relaxed);
+    processor_interface.m_fifo_cpu_end = fifo.CPEnd.load(std::memory_order_relaxed);
   }
 
   // If the game is running close to overflowing, make the exception checking more frequent.
@@ -405,7 +410,7 @@ void CommandProcessorManager::GatherPipeBursted(Core::System& system)
 
   fifo.CPReadWriteDistance.fetch_add(GPFifo::GATHER_PIPE_SIZE, std::memory_order_seq_cst);
 
-  Fifo::RunGpu();
+  system.GetFifo().RunGpu(system);
 
   ASSERT_MSG(COMMANDPROCESSOR,
              fifo.CPReadWriteDistance.load(std::memory_order_relaxed) <=
@@ -416,13 +421,13 @@ void CommandProcessorManager::GatherPipeBursted(Core::System& system)
   // check if we are in sync
   ASSERT_MSG(COMMANDPROCESSOR,
              fifo.CPWritePointer.load(std::memory_order_relaxed) ==
-                 ProcessorInterface::Fifo_CPUWritePointer,
+                 processor_interface.m_fifo_cpu_write_pointer,
              "FIFOs linked but out of sync");
   ASSERT_MSG(COMMANDPROCESSOR,
-             fifo.CPBase.load(std::memory_order_relaxed) == ProcessorInterface::Fifo_CPUBase,
+             fifo.CPBase.load(std::memory_order_relaxed) == processor_interface.m_fifo_cpu_base,
              "FIFOs linked but out of sync");
   ASSERT_MSG(COMMANDPROCESSOR,
-             fifo.CPEnd.load(std::memory_order_relaxed) == ProcessorInterface::Fifo_CPUEnd,
+             fifo.CPEnd.load(std::memory_order_relaxed) == processor_interface.m_fifo_cpu_end,
              "FIFOs linked but out of sync");
 }
 
@@ -432,22 +437,22 @@ void CommandProcessorManager::UpdateInterrupts(Core::System& system, u64 userdat
   {
     m_interrupt_set.Set();
     DEBUG_LOG_FMT(COMMANDPROCESSOR, "Interrupt set");
-    ProcessorInterface::SetInterrupt(INT_CAUSE_CP, true);
+    system.GetProcessorInterface().SetInterrupt(INT_CAUSE_CP, true);
   }
   else
   {
     m_interrupt_set.Clear();
     DEBUG_LOG_FMT(COMMANDPROCESSOR, "Interrupt cleared");
-    ProcessorInterface::SetInterrupt(INT_CAUSE_CP, false);
+    system.GetProcessorInterface().SetInterrupt(INT_CAUSE_CP, false);
   }
   system.GetCoreTiming().ForceExceptionCheck(0);
   m_interrupt_waiting.Clear();
-  Fifo::RunGpu();
+  system.GetFifo().RunGpu(system);
 }
 
 void CommandProcessorManager::UpdateInterruptsFromVideoBackend(Core::System& system, u64 userdata)
 {
-  if (!Fifo::UseDeterministicGPUThread())
+  if (!system.GetFifo().UseDeterministicGPUThread())
   {
     system.GetCoreTiming().ScheduleEvent(0, m_event_type_update_interrupts, userdata,
                                          CoreTiming::FromThread::NON_CPU);
@@ -563,7 +568,7 @@ void CommandProcessorManager::SetCPStatusFromCPU(Core::System& system)
       {
         m_interrupt_set.Set(interrupt);
         DEBUG_LOG_FMT(COMMANDPROCESSOR, "Interrupt set");
-        ProcessorInterface::SetInterrupt(INT_CAUSE_CP, interrupt);
+        system.GetProcessorInterface().SetInterrupt(INT_CAUSE_CP, interrupt);
       }
     }
     else
@@ -573,7 +578,7 @@ void CommandProcessorManager::SetCPStatusFromCPU(Core::System& system)
   }
 }
 
-void CommandProcessorManager::SetCpStatusRegister()
+void CommandProcessorManager::SetCpStatusRegister(Core::System& system)
 {
   const auto& fifo = m_fifo;
 
@@ -583,7 +588,7 @@ void CommandProcessorManager::SetCpStatusRegister()
                              (fifo.CPReadPointer.load(std::memory_order_relaxed) ==
                               fifo.CPWritePointer.load(std::memory_order_relaxed));
   m_cp_status_reg.CommandIdle = !fifo.CPReadWriteDistance.load(std::memory_order_relaxed) ||
-                                Fifo::AtBreakpoint() ||
+                                Fifo::AtBreakpoint(system) ||
                                 !fifo.bFF_GPReadEnable.load(std::memory_order_relaxed);
   m_cp_status_reg.UnderflowLoWatermark = fifo.bFF_LoWatermark.load(std::memory_order_relaxed);
   m_cp_status_reg.OverflowHiWatermark = fifo.bFF_HiWatermark.load(std::memory_order_relaxed);
@@ -597,7 +602,7 @@ void CommandProcessorManager::SetCpStatusRegister()
                 m_cp_status_reg.UnderflowLoWatermark ? "ON" : "OFF");
 }
 
-void CommandProcessorManager::SetCpControlRegister()
+void CommandProcessorManager::SetCpControlRegister(Core::System& system)
 {
   auto& fifo = m_fifo;
 
@@ -610,7 +615,7 @@ void CommandProcessorManager::SetCpControlRegister()
   if (fifo.bFF_GPReadEnable.load(std::memory_order_relaxed) && !m_cp_ctrl_reg.GPReadEnable)
   {
     fifo.bFF_GPReadEnable.store(m_cp_ctrl_reg.GPReadEnable, std::memory_order_relaxed);
-    Fifo::FlushGpu();
+    system.GetFifo().FlushGpu(system);
   }
   else
   {
@@ -632,7 +637,8 @@ void CommandProcessorManager::SetCpClearRegister()
 {
 }
 
-void CommandProcessorManager::HandleUnknownOpcode(u8 cmd_byte, const u8* buffer, bool preprocess)
+void CommandProcessorManager::HandleUnknownOpcode(Core::System& system, u8 cmd_byte,
+                                                  const u8* buffer, bool preprocess)
 {
   const auto& fifo = m_fifo;
 
@@ -660,6 +666,7 @@ void CommandProcessorManager::HandleUnknownOpcode(u8 cmd_byte, const u8* buffer,
   // PC and LR are meaningless when using the fifoplayer, and will generally not be helpful if the
   // unknown opcode is inside of a display list.  Also note that the changes in GPFifo.h are not
   // accurate and may introduce timing issues.
+  const auto& ppc_state = system.GetPPCState();
   GENERIC_LOG_FMT(
       Common::Log::LogType::VIDEO, log_level,
       "FIFO: Unknown Opcode {:#04x} @ {}, preprocessing = {}, CPBase: {:#010x}, CPEnd: "
@@ -681,7 +688,8 @@ void CommandProcessorManager::HandleUnknownOpcode(u8 cmd_byte, const u8* buffer,
       fifo.bFF_Breakpoint.load(std::memory_order_relaxed) ? "true" : "false",
       fifo.bFF_GPLinkEnable.load(std::memory_order_relaxed) ? "true" : "false",
       fifo.bFF_HiWatermarkInt.load(std::memory_order_relaxed) ? "true" : "false",
-      fifo.bFF_LoWatermarkInt.load(std::memory_order_relaxed) ? "true" : "false", PC, LR);
+      fifo.bFF_LoWatermarkInt.load(std::memory_order_relaxed) ? "true" : "false", ppc_state.pc,
+      LR(ppc_state));
 
   if (!m_is_fifo_error_seen && !suppress_panic_alert)
   {
