@@ -10,7 +10,8 @@
 
 namespace GraphicsModActionFactory
 {
-std::unique_ptr<GraphicsModAction> Create(std::string_view name, const picojson::value& json_data)
+std::unique_ptr<GraphicsModAction> Create(std::string_view name, const picojson::value& json_data,
+                                          std::string_view path)
 {
   if (name == "print")
   {
